from __future__ import annotations

from typing import TYPE_CHECKING

from django_spire.contrib.ordering.exceptions import OrderingMixinGroupError
from django_spire.contrib.ordering.validators import OrderingMixinValidator
from django_spire.contrib.service import BaseDjangoModelService

if TYPE_CHECKING:
    from django.db.models import Model, QuerySet
    from django_spire.contrib.ordering.mixins import OrderingModelMixin


class OrderingProcessorService(BaseDjangoModelService['OrderingModelMixin']):
    obj: OrderingModelMixin

    def _reorder_objects(self, objects: QuerySet[Model]):
        for index, item in enumerate(objects):
            item.order = index

        self.obj_class.objects.bulk_update(objects, ['order'])

    def _reorder_destination_and_origin_objects(
        self,
        destination_objects: QuerySet[Model],
        origin_objects: QuerySet[Model],
        insert_position: int,
    ):
        self._reorder_objects(origin_objects)

        # Forces destination objects to refresh in the event they overlap the origin objects
        destination_objects = destination_objects.all().order_by('order')

        for index, item in enumerate(destination_objects):
            if item.order >= insert_position:
                item.order = index + 1

        self.obj_class.objects.bulk_update(destination_objects, ['order'])

    def move_to_position(
        self,
        destination_objects: QuerySet[Model],
        position: int,
        origin_objects: QuerySet[Model] = None,
    ):
        # If origin objects not passed, assume origin objects are same as destination
        if origin_objects is None:
            origin_objects = destination_objects

        ordering_mixin_validator = OrderingMixinValidator(
            destination_objects=destination_objects,
            position=position,
            obj=self.obj,
            origin_objects=origin_objects,
        )

        if not ordering_mixin_validator.validate():
            message = 'Ordering validation failed.'

            raise OrderingMixinGroupError(
                message,
                ordering_mixin_validator.errors
            )

        destination_objects = destination_objects.exclude(pk=self.obj.pk).order_by('order')
        origin_objects = origin_objects.exclude(pk=self.obj.pk).order_by('order')

        self._reorder_destination_and_origin_objects(
            destination_objects=destination_objects,
            origin_objects=origin_objects,
            insert_position=position,
        )

        self.obj.order = position
        self.obj.save(update_fields=['order'])

    def remove_from_objects(
        self,
        destination_objects: QuerySet[Model],
    ):
        ordering_mixin_validator = OrderingMixinValidator(
            destination_objects=destination_objects,
            position=0,
            obj=self.obj,
            origin_objects=destination_objects,
        )

        if not ordering_mixin_validator.validate():
            message = 'Ordering validation failed.'

            raise ExceptionGroup(
                message,
                ordering_mixin_validator.errors
            )

        destination_objects = destination_objects.exclude(pk=self.obj.pk).order_by('order')

        self._reorder_objects(objects=destination_objects)



from __future__ import annotations

import json

from django_spire.core.tests.test_cases import BaseTestCase
from django_spire.file.widgets import MultipleWidget, SingleFileWidget


class MultipleWidgetTests(BaseTestCase):
    def setUp(self):
        super().setUp()

        self.widget = MultipleWidget()

    def test_needs_multipart_form(self):
        assert self.widget.needs_multipart_form is True

    def test_template_name(self):
        assert self.widget.template_name == 'django_spire/file/widget/multiple_file_widget.html'

    def test_value_from_datadict(self):
        data = {
            'files_data': json.dumps([{'id': 1, 'name': 'test'}])
        }

        result = self.widget.value_from_datadict(data, None, 'files')

        assert result == [{'id': 1, 'name': 'test'}]

    def test_value_from_datadict_empty(self):
        data = {
            'files_data': json.dumps([])
        }

        result = self.widget.value_from_datadict(data, None, 'files')

        assert result == []

    def test_value_from_datadict_multiple_files(self):
        data = {
            'files_data': json.dumps([
                {'id': 1, 'name': 'file1'},
                {'id': 2, 'name': 'file2'},
            ])
        }

        result = self.widget.value_from_datadict(data, None, 'files')

        assert len(result) == 2
        assert result[0]['name'] == 'file1'
        assert result[1]['name'] == 'file2'


class SingleFileWidgetTests(BaseTestCase):
    def setUp(self):
        super().setUp()

        self.widget = SingleFileWidget()

    def test_needs_multipart_form(self):
        assert self.widget.needs_multipart_form is True

    def test_template_name(self):
        assert self.widget.template_name == 'django_spire/file/widget/single_file_widget.html'

    def test_value_from_datadict(self):
        data = {
            'file_data': json.dumps({'id': 1, 'name': 'test'})
        }

        result = self.widget.value_from_datadict(data, None, 'file')

        assert result == {'id': 1, 'name': 'test'}

    def test_value_from_datadict_null(self):
        data = {
            'file_data': json.dumps(None)
        }

        result = self.widget.value_from_datadict(data, None, 'file')

        assert result is None



from __future__ import annotations

from typing import TYPE_CHECKING

from django.http import HttpResponseRedirect
from django.shortcuts import get_object_or_404
from django.urls import reverse

import django_glue as dg

from django_spire.auth.group import models, forms
from django_spire.auth.group.utils import set_group_users
from django_spire.auth.permissions.decorators import permission_required
from django_spire.auth.user.models import AuthUser
from django_spire.contrib.form.confirmation_forms import DeleteConfirmationForm
from django_spire.contrib.form.utils import show_form_errors
from django_spire.contrib.generic_views import portal_views
from django_spire.core.shortcuts import get_object_or_null_obj
from django_spire.history.activity.utils import add_form_activity

if TYPE_CHECKING:
    from django.core.handlers.wsgi import WSGIRequest
    from django.template.response import TemplateResponse

    from django_spire.contrib.breadcrumb.models import Breadcrumbs


@permission_required('django_spire_auth_group.change_authgroup')
def form_view(
    request: WSGIRequest,
    pk: int = 0
) -> TemplateResponse | HttpResponseRedirect:
    group = get_object_or_null_obj(models.AuthGroup, pk=pk)

    dg.glue_model_object(request, 'group', group)

    if request.method == 'POST':
        form = forms.GroupForm(request.POST, instance=group)

        if form.is_valid():
            group = form.save()
            add_form_activity(group, pk, request.user)

            return_url = reverse('django_spire:auth:group:page:list')
            return HttpResponseRedirect(return_url)

        show_form_errors(request, form)

    form = forms.GroupForm(instance=group)

    context_data = {'group': group}

    return portal_views.model_form_view(
        request,
        form=form,
        context_data=context_data,
        template='django_spire/auth/group/page/form_page.html',
        obj=group,
    )


@permission_required('django_spire_auth_group.add_authgroup')
def user_form_view(
    request: WSGIRequest,
    pk: int
) -> TemplateResponse | HttpResponseRedirect:
    group = get_object_or_404(models.AuthGroup, pk=pk)
    user_choices = AuthUser.services.get_user_choices()

    selected_user_ids = list(
        AuthUser.objects
        .filter(groups=group)
        .values_list('id', flat=True)
        .distinct()
    )

    if request.method == 'POST':
        form = forms.GroupUserForm(data=request.POST)

        if form.is_valid():
            user_list = form.cleaned_data.get('users')
            set_group_users(group, user_list)

            group.add_activity(
                user=request.user,
                verb='added',
                information=(
                    f'{request.user.get_full_name()} added {len(user_list)} users to '
                    f'the group "{group.name}".'
                )
            )

            return_url = reverse('django_spire:auth:group:page:detail', kwargs={'pk': pk})
            return HttpResponseRedirect(return_url)

        show_form_errors(request, form)

    form = forms.GroupUserForm()

    def crumbs(breadcrumbs: Breadcrumbs) -> None:
        breadcrumbs.add_breadcrumb(name='Edit Users')

    context_data = {
        'group': group,
        'user_choices': user_choices,
        'selected_user_ids': selected_user_ids
    }

    return portal_views.form_view(
        request,
        form=form,
        obj=group,
        template='django_spire/auth/group/page/group_user_form_page.html',
        context_data=context_data,
        breadcrumbs_func=crumbs,
    )


@permission_required('django_spire_auth_group.delete_authgroup')
def delete_form_view(request: WSGIRequest, pk: int) -> TemplateResponse:
    group = get_object_or_404(models.AuthGroup, pk=pk)
    return_url = reverse('django_spire:auth:group:page:list')

    return portal_views.delete_form_view(
        request,
        obj=group,
        delete_func=group.delete,
        activity_func=lambda: None,
        return_url=return_url
    )


@permission_required('django_spire_auth_group.delete_authgroup')
def group_remove_user_form_view(
    request: WSGIRequest,
    group_pk: int,
    pk: int
) -> HttpResponseRedirect | TemplateResponse:
    group = get_object_or_404(models.AuthGroup, pk=group_pk)
    user = get_object_or_404(AuthUser, pk=pk)

    if request.method == 'POST':
        form = DeleteConfirmationForm(request.POST, obj=user)

        if form.is_valid():
            user.groups.remove(group)

            group.add_activity(
                user=request.user,
                verb='removed',
                information=(
                    f'{request.user.get_full_name()} removed {user.get_full_name()} '
                    f'from the group "{group.name}".'
                )
            )

            return HttpResponseRedirect(
                reverse(
                    'django_spire:auth:group:page:detail',
                    kwargs={'pk': group_pk}
                )
            )

    form = DeleteConfirmationForm(request.GET, obj=user)

    def get_breadcrumbs(breadcrumbs: Breadcrumbs) -> None:
        breadcrumbs = group.breadcrumbs()
        breadcrumbs.add_breadcrumb(name=user.get_full_name())
        breadcrumbs.add_breadcrumb(name='Remove')

    return portal_views.form_view(
        request,
        form=form,
        verb=f'remove {user.get_full_name()} from',
        obj=group,
        breadcrumbs_func=get_breadcrumbs,
        template = 'django_spire/page/delete_confirmation_form_page.html'
    )



from __future__ import annotations

from django.test import TestCase

from django_spire import settings


class TestDefaultSettings(TestCase):
    def test_ai_chat_routers_contains_spire(self) -> None:
        assert 'SPIRE' in settings.DJANGO_SPIRE_AI_CHAT_ROUTERS

    def test_ai_default_chat_router(self) -> None:
        assert settings.DJANGO_SPIRE_AI_DEFAULT_CHAT_ROUTER == 'SPIRE'

    def test_ai_intent_chat_routers_contains_knowledge_search(self) -> None:
        assert 'KNOWLEDGE_SEARCH' in settings.DJANGO_SPIRE_AI_INTENT_CHAT_ROUTERS

    def test_ai_intent_chat_routers_structure(self) -> None:
        knowledge_search = settings.DJANGO_SPIRE_AI_INTENT_CHAT_ROUTERS['KNOWLEDGE_SEARCH']

        assert 'INTENT_DESCRIPTION' in knowledge_search
        assert 'REQUIRED_PERMISSION' in knowledge_search
        assert 'CHAT_ROUTER' in knowledge_search

    def test_ai_persona_name(self) -> None:
        assert settings.DJANGO_SPIRE_AI_PERSONA_NAME == 'AI Assistant'

    def test_auth_controllers_contains_ai_chat(self) -> None:
        assert 'ai_chat' in settings.DJANGO_SPIRE_AUTH_CONTROLLERS

    def test_auth_controllers_contains_help_desk(self) -> None:
        assert 'help_desk' in settings.DJANGO_SPIRE_AUTH_CONTROLLERS

    def test_auth_controllers_contains_knowledge(self) -> None:
        assert 'knowledge' in settings.DJANGO_SPIRE_AUTH_CONTROLLERS

    def test_auth_controllers_is_dict(self) -> None:
        assert isinstance(settings.DJANGO_SPIRE_AUTH_CONTROLLERS, dict)

    def test_default_theme(self) -> None:
        assert settings.DJANGO_SPIRE_DEFAULT_THEME == 'default-light'

    def test_theme_path_contains_placeholders(self) -> None:
        assert '{family}' in settings.DJANGO_SPIRE_THEME_PATH
        assert '{mode}' in settings.DJANGO_SPIRE_THEME_PATH



from __future__ import annotations

from django_spire.core.tests.test_cases import BaseTestCase
from django_spire.knowledge.entry.version.choices import EntryVersionStatusChoices
from django_spire.knowledge.entry.version.tests.factories import create_test_entry_version


class EntryVersionModelTests(BaseTestCase):
    def setUp(self):
        super().setUp()
        self.entry_version = create_test_entry_version()

    def test_is_published_false_when_draft(self):
        self.entry_version.status = EntryVersionStatusChoices.DRAFT
        assert self.entry_version.is_published() is False

    def test_is_published_true_when_published(self):
        self.entry_version.status = EntryVersionStatusChoices.PUBLISHED
        assert self.entry_version.is_published() is True

    def test_is_published_false_when_archived(self):
        self.entry_version.status = EntryVersionStatusChoices.ARCHIVED
        assert self.entry_version.is_published() is False



from __future__ import annotations

import json

from django.test import override_settings

from django_spire.core.tests.test_cases import BaseTestCase
from django_spire.file.fields import MultipleFileField, SingleFileField
from django_spire.file.models import File
from django_spire.file.tests.factories import create_test_file
from django_spire.file.widgets import MultipleWidget, SingleFileWidget


STORAGES_OVERRIDE = {
    'default': {
        'BACKEND': 'django.core.files.storage.FileSystemStorage',
    },
    'staticfiles': {
        'BACKEND': 'django.contrib.staticfiles.storage.StaticFilesStorage',
    },
}


@override_settings(STORAGES=STORAGES_OVERRIDE)
class MultipleFileFieldTests(BaseTestCase):
    def setUp(self):
        super().setUp()

        self.field = MultipleFileField()

    def test_widget_is_multiple_widget(self):
        assert isinstance(self.field.widget, MultipleWidget)

    def test_prepare_value_with_files(self):
        file1 = create_test_file(name='file1')
        file2 = create_test_file(name='file2')

        result = self.field.prepare_value([file1, file2])
        parsed = json.loads(result)

        assert len(parsed) == 2

    def test_prepare_value_with_none(self):
        result = self.field.prepare_value(None)

        assert result == '[]'

    def test_prepare_value_with_empty_list(self):
        result = self.field.prepare_value([])

        assert result == '[]'

    def test_prepare_value_contains_file_data(self):
        file = create_test_file(name='test')

        result = self.field.prepare_value([file])
        parsed = json.loads(result)

        assert parsed[0]['name'] == 'test'
        assert 'id' in parsed[0]
        assert 'url' in parsed[0]

    def test_clean_returns_data(self):
        data = {'test': 'value'}

        result = self.field.clean(data)

        assert result == data


@override_settings(STORAGES=STORAGES_OVERRIDE)
class SingleFileFieldTests(BaseTestCase):
    def setUp(self):
        super().setUp()

        self.field = SingleFileField()

    def test_widget_is_single_file_widget(self):
        assert isinstance(self.field.widget, SingleFileWidget)

    def test_prepare_value_with_file(self):
        file = create_test_file(name='test')

        result = self.field.prepare_value(file)
        parsed = json.loads(result)

        assert parsed['name'] == 'test'
        assert 'id' in parsed
        assert 'url' in parsed

    def test_prepare_value_with_none(self):
        result = self.field.prepare_value(None)

        assert result == 'null'

    def test_prepare_value_with_queryset(self):
        file = create_test_file(name='queryset_file')

        result = self.field.prepare_value(File.objects.filter(pk=file.pk))
        parsed = json.loads(result)

        assert parsed['name'] == 'queryset_file'

    def test_prepare_value_with_empty_queryset(self):
        result = self.field.prepare_value(File.objects.none())

        assert result == 'null'

    def test_clean_returns_data(self):
        data = {'test': 'value'}

        result = self.field.clean(data)

        assert result == data



from __future__ import annotations

import django_glue as dg

from typing import TYPE_CHECKING

from django.shortcuts import redirect, get_object_or_404
from django.urls import reverse

from django_spire.auth.controller.controller import AppAuthController
from django_spire.contrib.form.utils import show_form_errors
from django_spire.contrib.generic_views import portal_views
from django_spire.help_desk import forms
from django_spire.help_desk.models import HelpDeskTicket

if TYPE_CHECKING:
    from django.core.handlers.wsgi import WSGIRequest
    from django.template.response import TemplateResponse


@AppAuthController('help_desk').permission_required('can_add')
def ticket_create_form_view(request: WSGIRequest) -> TemplateResponse:
    ticket = HelpDeskTicket()

    dg.glue_model_object(request, 'ticket', ticket)

    if request.method == 'POST':
        form = forms.HelpDeskTicketCreateForm(request.POST)

        if form.is_valid():
            ticket.services.create(
                created_by=request.user,
                **form.cleaned_data
            )
            return redirect(reverse('django_spire:help_desk:page:list'))

        show_form_errors(request, form)

    else:
        form = forms.HelpDeskTicketCreateForm(instance=ticket)

    return portal_views.form_view(
        request,
        form=form,
        template='django_spire/help_desk/page/ticket_form_page.html',
        verb='Create',
        obj=ticket,
        context_data={
            'form_action_url': reverse('django_spire:help_desk:form:create'),
        }
    )


@AppAuthController('help_desk').permission_required('can_change')
def ticket_update_form_view(request: WSGIRequest, pk: int) -> TemplateResponse:
    ticket = get_object_or_404(HelpDeskTicket, pk=pk)

    dg.glue_model_object(request, 'ticket', ticket)

    if request.method == 'POST':
        form = forms.HelpDeskTicketUpdateForm(request.POST, instance=ticket)

        if form.is_valid():
            ticket.services.save_model_obj(**form.cleaned_data)
            return redirect(reverse('django_spire:help_desk:page:list'))

        show_form_errors(request, form)
    else:
        form = forms.HelpDeskTicketUpdateForm(instance=ticket)

    return portal_views.model_form_view(
        request=request,
        obj=ticket,
        template='django_spire/help_desk/page/ticket_form_page.html',
        form=form,
        context_data={
            'ticket': ticket,
            'form_action_url': (
                reverse('django_spire:help_desk:form:update', kwargs={'pk': ticket.pk})
            ),
        }
    )



from __future__ import annotations

from unittest.mock import MagicMock, patch

from django.contrib.auth.models import User
from django.http import HttpResponseRedirect
from django.template.response import TemplateResponse
from django.test import RequestFactory, TestCase

from django_spire.contrib.breadcrumb.breadcrumbs import Breadcrumbs
from django_spire.contrib.generic_views.modal_views import dispatch_modal_delete_form_content
from django_spire.contrib.generic_views.portal_views import (
    delete_form_view,
    detail_view,
    form_view,
    infinite_scrolling_view,
    list_view,
    model_form_view,
    template_view,
)


class TestDispatchModalDeleteFormContent(TestCase):
    def setUp(self) -> None:
        self.factory = RequestFactory()
        self.user = User.objects.create_user(
            username='testuser',
            password='testpass'  # noqa: S106
        )
        self.obj = MagicMock()
        self.obj._meta.model._meta.verbose_name = 'Test Model'
        self.obj.__str__ = MagicMock(return_value='Test Object')

    def test_get_request_returns_template_response(self) -> None:
        request = self.factory.get('/')
        request.user = self.user

        response = dispatch_modal_delete_form_content(
            request,
            obj=self.obj,
            form_action='/delete/'
        )

        assert isinstance(response, TemplateResponse)

    def test_get_request_context_contains_form_data(self) -> None:
        request = self.factory.get('/')
        request.user = self.user

        response = dispatch_modal_delete_form_content(
            request,
            obj=self.obj,
            form_action='/delete/'
        )

        assert 'form_title' in response.context_data
        assert 'form_action' in response.context_data
        assert 'form_description' in response.context_data

    def test_post_request_returns_redirect(self) -> None:
        request = self.factory.post('/', data={'should_delete': True})
        request.user = self.user

        del self.obj.add_activity

        with patch('django_spire.contrib.generic_views.modal_views.safe_redirect_url', return_value='/'):
            response = dispatch_modal_delete_form_content(
                request,
                obj=self.obj,
                form_action='/delete/'
            )

        assert isinstance(response, HttpResponseRedirect)

    def test_custom_verbs(self) -> None:
        request = self.factory.get('/')
        request.user = self.user

        response = dispatch_modal_delete_form_content(
            request,
            obj=self.obj,
            form_action='/archive/',
            verbs=('archive', 'archived')
        )

        assert 'Archive' in response.context_data['form_title']

    def test_custom_return_url(self) -> None:
        request = self.factory.post('/', data={'should_delete': True})
        request.user = self.user

        del self.obj.add_activity

        response = dispatch_modal_delete_form_content(
            request,
            obj=self.obj,
            form_action='/delete/',
            return_url='/custom/'
        )

        assert response.url == '/custom/'


class TestDetailView(TestCase):
    def setUp(self) -> None:
        self.factory = RequestFactory()
        self.obj = MagicMock()
        self.obj._meta.model._meta.verbose_name = 'Test Model'
        self.obj.__str__ = MagicMock(return_value='Test Object')
        del self.obj.breadcrumbs

    def test_returns_template_response(self) -> None:
        request = self.factory.get('/')

        response = detail_view(
            request,
            obj=self.obj,
            template='test_template.html'
        )

        assert isinstance(response, TemplateResponse)

    def test_context_contains_page_data(self) -> None:
        request = self.factory.get('/')

        response = detail_view(
            request,
            obj=self.obj,
            template='test_template.html'
        )

        assert 'page_title' in response.context_data
        assert 'page_description' in response.context_data
        assert 'breadcrumbs' in response.context_data

    def test_custom_breadcrumbs_func(self) -> None:
        request = self.factory.get('/')

        def breadcrumbs_func(breadcrumbs: Breadcrumbs) -> None:
            breadcrumbs.add_breadcrumb('Custom', '/custom/')

        response = detail_view(
            request,
            obj=self.obj,
            template='test_template.html',
            breadcrumbs_func=breadcrumbs_func
        )

        assert len(response.context_data['breadcrumbs']) == 1


class TestDeleteFormView(TestCase):
    def setUp(self) -> None:
        self.factory = RequestFactory()
        self.user = User.objects.create_user(
            username='testuser',
            password='testpass'  # noqa: S106
        )
        self.obj = MagicMock()
        self.obj._meta.model._meta.verbose_name = 'Test Model'
        self.obj.__str__ = MagicMock(return_value='Test Object')
        del self.obj.breadcrumbs

    def test_get_request_returns_template_response(self) -> None:
        request = self.factory.get('/')
        request.user = self.user

        response = delete_form_view(
            request,
            obj=self.obj,
            return_url='/'
        )

        assert isinstance(response, TemplateResponse)

    def test_post_with_should_delete_redirects(self) -> None:
        request = self.factory.post('/', data={'should_delete': True})
        request.user = self.user

        del self.obj.add_activity

        response = delete_form_view(
            request,
            obj=self.obj,
            return_url='/list/'
        )

        assert isinstance(response, HttpResponseRedirect)
        assert response.url == '/list/'

    def test_post_calls_set_deleted(self) -> None:
        request = self.factory.post('/', data={'should_delete': True})
        request.user = self.user

        del self.obj.add_activity

        delete_form_view(
            request,
            obj=self.obj,
            return_url='/'
        )

        self.obj.set_deleted.assert_called_once()


class TestInfiniteScrollingView(TestCase):
    def setUp(self) -> None:
        self.factory = RequestFactory()

    def test_returns_template_response(self) -> None:
        request = self.factory.get('/')

        response = infinite_scrolling_view(
            request,
            queryset=[1, 2, 3, 4, 5],
            queryset_name='items',
            template='test_template.html'
        )

        assert isinstance(response, TemplateResponse)

    def test_pagination_first_page(self) -> None:
        request = self.factory.get('/', {'page': '1'})

        response = infinite_scrolling_view(
            request,
            queryset=list(range(100)),
            queryset_name='items',
            template='test_template.html'
        )

        assert len(response.context_data['items']) == 50
        assert response.context_data['has_next'] is True

    def test_pagination_last_page(self) -> None:
        request = self.factory.get('/', {'page': '2'})

        response = infinite_scrolling_view(
            request,
            queryset=list(range(60)),
            queryset_name='items',
            template='test_template.html'
        )

        assert len(response.context_data['items']) == 10
        assert response.context_data['has_next'] is False

    def test_custom_batch_size(self) -> None:
        request = self.factory.get('/', {'batch_size': '10'})

        response = infinite_scrolling_view(
            request,
            queryset=list(range(25)),
            queryset_name='items',
            template='test_template.html'
        )

        assert len(response.context_data['items']) == 10
        assert response.context_data['batch_size'] == 10

    def test_total_count_in_context(self) -> None:
        request = self.factory.get('/')

        response = infinite_scrolling_view(
            request,
            queryset=list(range(75)),
            queryset_name='items',
            template='test_template.html'
        )

        assert response.context_data['total_count'] == 75


class TestListView(TestCase):
    def setUp(self) -> None:
        self.factory = RequestFactory()
        self.model = MagicMock()
        self.model._meta.verbose_name = 'Test Model'

    def test_returns_template_response(self) -> None:
        request = self.factory.get('/')

        response = list_view(
            request,
            model=self.model,
            template='test_template.html'
        )

        assert isinstance(response, TemplateResponse)

    def test_context_contains_page_data(self) -> None:
        request = self.factory.get('/')

        response = list_view(
            request,
            model=self.model,
            template='test_template.html'
        )

        assert response.context_data['page_title'] == 'Test Model'
        assert response.context_data['page_description'] == 'List View'
        assert 'breadcrumbs' in response.context_data

    def test_custom_breadcrumbs_func(self) -> None:
        request = self.factory.get('/')

        def breadcrumbs_func(breadcrumbs: Breadcrumbs) -> None:
            breadcrumbs.add_breadcrumb('Custom List', '/list/')

        response = list_view(
            request,
            model=self.model,
            template='test_template.html',
            breadcrumbs_func=breadcrumbs_func
        )

        assert len(response.context_data['breadcrumbs']) == 1


class TestFormView(TestCase):
    def setUp(self) -> None:
        self.factory = RequestFactory()
        self.obj = MagicMock()
        self.obj._meta.model._meta.verbose_name = 'Test Model'
        self.obj.__str__ = MagicMock(return_value='Test Object')
        self.obj.pk = 1
        del self.obj.breadcrumbs
        self.form = MagicMock()

    def test_returns_template_response(self) -> None:
        request = self.factory.get('/')

        response = form_view(
            request,
            form=self.form,
            obj=self.obj
        )

        assert isinstance(response, TemplateResponse)

    def test_edit_verb_for_existing_object(self) -> None:
        request = self.factory.get('/')

        response = form_view(
            request,
            form=self.form,
            obj=self.obj
        )

        assert 'Edit' in response.context_data['form_title']

    def test_create_verb_for_new_object(self) -> None:
        request = self.factory.get('/')
        self.obj.pk = None

        response = form_view(
            request,
            form=self.form,
            obj=self.obj
        )

        assert 'Create' in response.context_data['form_title']

    def test_custom_verb(self) -> None:
        request = self.factory.get('/')

        response = form_view(
            request,
            form=self.form,
            obj=self.obj,
            verb='Update'
        )

        assert 'Update' in response.context_data['form_title']


class TestModelFormView(TestCase):
    def setUp(self) -> None:
        self.factory = RequestFactory()
        self.obj = MagicMock()
        self.obj._meta.model._meta.verbose_name = 'Test Model'
        self.obj.__str__ = MagicMock(return_value='Test Object')
        self.obj.pk = 1
        del self.obj.breadcrumbs
        self.form = MagicMock()

    def test_returns_template_response(self) -> None:
        request = self.factory.get('/')

        response = model_form_view(
            request,
            form=self.form,
            obj=self.obj
        )

        assert isinstance(response, TemplateResponse)

    def test_delegates_to_form_view(self) -> None:
        request = self.factory.get('/')

        response = model_form_view(
            request,
            form=self.form,
            obj=self.obj
        )

        assert 'form' in response.context_data
        assert 'breadcrumbs' in response.context_data


class TestTemplateView(TestCase):
    def setUp(self) -> None:
        self.factory = RequestFactory()

    def test_returns_template_response(self) -> None:
        request = self.factory.get('/')
        breadcrumbs = Breadcrumbs()

        response = template_view(
            request,
            page_title='Test Page',
            page_description='Test Description',
            breadcrumbs=breadcrumbs,
            template='test_template.html'
        )

        assert isinstance(response, TemplateResponse)

    def test_context_contains_page_data(self) -> None:
        request = self.factory.get('/')
        breadcrumbs = Breadcrumbs()
        breadcrumbs.add_breadcrumb('Home', '/')

        response = template_view(
            request,
            page_title='Test Page',
            page_description='Test Description',
            breadcrumbs=breadcrumbs,
            template='test_template.html'
        )

        assert response.context_data['page_title'] == 'Test Page'
        assert response.context_data['page_description'] == 'Test Description'
        assert len(response.context_data['breadcrumbs']) == 1

    def test_custom_context_data(self) -> None:
        request = self.factory.get('/')
        breadcrumbs = Breadcrumbs()

        response = template_view(
            request,
            page_title='Test Page',
            page_description='Test Description',
            breadcrumbs=breadcrumbs,
            template='test_template.html',
            context_data={'custom_key': 'custom_value'}
        )

        assert response.context_data['custom_key'] == 'custom_value'



from __future__ import annotations

from typing import TypeVar, TYPE_CHECKING

from django.db.models.base import Model
from django.db.models import ForeignKey

from django_spire.contrib.seeding.field.callable import CallableFieldSeeder
from django_spire.contrib.seeding.field.custom import CustomFieldSeeder
from django_spire.contrib.seeding.field.django.seeder import DjangoFieldFakerSeeder, DjangoFieldLlmSeeder
from django_spire.contrib.seeding.field.static import StaticFieldSeeder
from django_spire.contrib.seeding.model.base import BaseModelSeeder
from django_spire.contrib.seeding.model.django.config import DjangoModelFieldsConfig

if TYPE_CHECKING:
    from dandy import Prompt


TypeModel = TypeVar('TypeModel', bound=Model)


class DjangoModelSeeder(BaseModelSeeder):
    field_config_class = DjangoModelFieldsConfig

    model_class: type[Model]
    prompt: Prompt = None
    _field_seeders = [
        CustomFieldSeeder,
        CallableFieldSeeder,
        StaticFieldSeeder,
        DjangoFieldFakerSeeder,
        DjangoFieldLlmSeeder
    ]

    @classmethod
    def __init_subclass__(cls, **kwargs) -> None:
        super().__init_subclass__(**kwargs)

        if cls.model_class is None:
            message = 'Seeds must have a model class'
            raise ValueError(message)

        if cls.fields is None:
            message = 'Seeds must have fields'
            raise ValueError(message)

    @classmethod
    def field_names(cls) -> list[str]:
        # All foreign keys must be _id
        return [
            f.attname if isinstance(f, ForeignKey) else f.name
            for f in cls.model_class._meta.fields
        ]

    @classmethod
    def seed_database(
        cls,
        count: int = 1,
        fields: dict | None = None
    ) -> list[TypeModel]:
        model_objects = cls.seed(count, fields)
        return cls.model_class.objects.bulk_create(model_objects)



from __future__ import annotations

from django.db import connection, models


class DummyModel(models.Model):
    name = models.CharField(max_length=255, default='')

    class Meta:
        app_label = 'django_spire_core'
        db_table = 'django_spire_core_dummymodel'

    def __str__(self) -> str:
        return 'dummy'

    @classmethod
    def create_table(cls) -> None:
        vendor = connection.vendor

        with connection.cursor() as cursor:
            if vendor == 'sqlite':
                query = '''
                    CREATE TABLE IF NOT EXISTS django_spire_core_dummymodel (
                        id INTEGER PRIMARY KEY AUTOINCREMENT,
                        name VARCHAR(255) DEFAULT ''
                    )
                '''
            else:
                query = '''
                    CREATE TABLE IF NOT EXISTS django_spire_core_dummymodel (
                        id SERIAL PRIMARY KEY,
                        name VARCHAR(255) DEFAULT ''
                    )
                '''

            cursor.execute(query)

    @classmethod
    def drop_table(cls) -> None:
        with connection.cursor() as cursor:
            cursor.execute('DROP TABLE IF EXISTS django_spire_core_dummymodel')


class DummyModelMixin:
    @classmethod
    def setUpClass(cls) -> None:
        super().setUpClass()

        DummyModel.create_table()

    @classmethod
    def tearDownClass(cls) -> None:
        DummyModel.drop_table()
        super().tearDownClass()



from __future__ import annotations

from django_spire.core.tests.test_cases import BaseTestCase
from django_spire.knowledge.collection.tests.factories import create_test_collection
from django_spire.knowledge.entry.models import Entry
from django_spire.knowledge.entry.tests.factories import create_test_entry
from django_spire.knowledge.entry.version.tests.factories import create_test_entry_version


class EntryServiceTests(BaseTestCase):
    def setUp(self):
        super().setUp()
        self.collection = create_test_collection()
        self.entry = create_test_entry(collection=self.collection)

    def test_save_model_obj_creates_new_with_version(self):
        entry, created = Entry.services.save_model_obj(
            author=self.super_user,
            name='New Entry',
            collection=self.collection
        )
        assert created is True
        assert entry.name == 'New Entry'
        assert entry.current_version is not None

    def test_save_model_obj_updates_existing(self):
        entry_version = create_test_entry_version(entry=self.entry)
        self.entry.current_version = entry_version
        self.entry.save()

        self.entry.services.obj = self.entry
        updated_entry, created = self.entry.services.save_model_obj(
            author=self.super_user,
            name='Updated Name'
        )
        assert created is False
        assert updated_entry.name == 'Updated Name'


class EntryProcessorServiceTests(BaseTestCase):
    def setUp(self):
        super().setUp()
        self.collection = create_test_collection()
        self.entry = create_test_entry(collection=self.collection)

    def test_set_deleted(self):
        self.entry.services.processor.set_deleted()
        self.entry.refresh_from_db()
        assert self.entry.is_deleted is True


class EntryToolServiceTests(BaseTestCase):
    def setUp(self):
        super().setUp()
        self.collection = create_test_collection()
        self.entry = create_test_entry(collection=self.collection)

    def test_get_files_to_convert(self):
        result = Entry.services.tool.get_files_to_convert()
        assert result.count() == 0

    def test_get_files_to_convert_json(self):
        result = Entry.services.tool.get_files_to_convert_json()
        assert result == '[]'


class EntryTransformationServiceTests(BaseTestCase):
    def setUp(self):
        super().setUp()
        self.collection = create_test_collection()
        self.entry = create_test_entry(collection=self.collection)
        self.entry_version = create_test_entry_version(entry=self.entry)
        self.entry.current_version = self.entry_version
        self.entry.save()

    def test_to_dict(self):
        self.entry.services.transformation.obj = self.entry
        result = self.entry.services.transformation.to_dict()

        assert result['entry_id'] == self.entry.pk
        assert result['name'] == self.entry.name
        assert 'version_id' in result
        assert 'author' in result
        assert 'status' in result
        assert 'delete_url' in result
        assert 'edit_url' in result
        assert 'view_url' in result

    def test_queryset_to_navigation_list(self):
        result = Entry.services.transformation.queryset_to_navigation_list(
            queryset=Entry.objects.filter(pk=self.entry.pk)
        )
        assert len(result) == 1
        assert result[0]['entry_id'] == self.entry.pk



from __future__ import annotations

from django.contrib.contenttypes.fields import GenericRelation
from django.test import TestCase

from django_spire.history.activity.mixins import ActivityMixin
from django_spire.history.mixins import HistoryModelMixin
from django_spire.history.viewed.mixins import ViewedModelMixin


class TestActivityMixin(TestCase):
    def test_activities_field_exists(self) -> None:
        assert hasattr(ActivityMixin, 'activities')

    def test_activities_is_generic_relation(self) -> None:
        field = ActivityMixin._meta.get_field('activities')
        assert isinstance(field, GenericRelation)

    def test_add_activity_method_exists(self) -> None:
        assert hasattr(ActivityMixin, 'add_activity')
        assert callable(ActivityMixin.add_activity)

    def test_creator_property_exists(self) -> None:
        assert hasattr(ActivityMixin, 'creator')

    def test_is_abstract(self) -> None:
        assert ActivityMixin._meta.abstract is True


class TestHistoryModelMixin(TestCase):
    def test_created_datetime_field_exists(self) -> None:
        assert hasattr(HistoryModelMixin, 'created_datetime')

    def test_history_events_field_exists(self) -> None:
        assert hasattr(HistoryModelMixin, 'history_events')

    def test_history_events_is_generic_relation(self) -> None:
        field = HistoryModelMixin._meta.get_field('history_events')
        assert isinstance(field, GenericRelation)

    def test_is_abstract(self) -> None:
        assert HistoryModelMixin._meta.abstract is True

    def test_is_active_field_exists(self) -> None:
        assert hasattr(HistoryModelMixin, 'is_active')

    def test_is_deleted_field_exists(self) -> None:
        assert hasattr(HistoryModelMixin, 'is_deleted')

    def test_set_active_method_exists(self) -> None:
        assert hasattr(HistoryModelMixin, 'set_active')
        assert callable(HistoryModelMixin.set_active)

    def test_set_deleted_method_exists(self) -> None:
        assert hasattr(HistoryModelMixin, 'set_deleted')
        assert callable(HistoryModelMixin.set_deleted)

    def test_set_inactive_method_exists(self) -> None:
        assert hasattr(HistoryModelMixin, 'set_inactive')
        assert callable(HistoryModelMixin.set_inactive)

    def test_un_set_deleted_method_exists(self) -> None:
        assert hasattr(HistoryModelMixin, 'un_set_deleted')
        assert callable(HistoryModelMixin.un_set_deleted)


class TestViewedModelMixin(TestCase):
    def test_add_view_method_exists(self) -> None:
        assert hasattr(ViewedModelMixin, 'add_view')
        assert callable(ViewedModelMixin.add_view)

    def test_is_abstract(self) -> None:
        assert ViewedModelMixin._meta.abstract is True

    def test_is_viewed_method_exists(self) -> None:
        assert hasattr(ViewedModelMixin, 'is_viewed')
        assert callable(ViewedModelMixin.is_viewed)

    def test_views_field_exists(self) -> None:
        assert hasattr(ViewedModelMixin, 'views')

    def test_views_is_generic_relation(self) -> None:
        field = ViewedModelMixin._meta.get_field('views')
        assert isinstance(field, GenericRelation)



from __future__ import annotations

import re

from typing import TYPE_CHECKING

from django.core.management.base import CommandError

if TYPE_CHECKING:
    from django_spire.core.management.commands.spire_startapp_pkg.reporter import Reporter
    from django_spire.core.management.commands.spire_startapp_pkg.validator import AppValidator


class UserInputCollector:
    """
    Collects user input for Django app creation through an interactive wizard.

    This class guides users through a step-by-step process to gather all
    necessary configuration for creating a new Django app.
    """

    def __init__(self, reporter: Reporter, validator: AppValidator):
        """
        Initializes the collector with a reporter and validator.

        :param reporter: Reporter instance for displaying prompts and messages.
        :param validator: Validator for checking user input validity.
        """

        self.reporter = reporter
        self.validator = validator

    def collect_all_inputs(self) -> dict[str, str]:
        """
        Collects all required user inputs for app creation.

        Guides the user through an 8-step wizard to gather app path, names,
        labels, and configuration options.

        :return: Dictionary containing all collected user inputs.
        """

        self.reporter.write('\n[App Creation Wizard]\n\n', self.reporter.style_success)

        app_path = self._collect_app_path()
        components = app_path.split('.')

        app_name = self._collect_app_name(components)
        app_label = self._collect_app_label(components, app_name)
        model_name = self._collect_model_name(app_name)
        model_name_plural = self._collect_model_name_plural(model_name)
        db_table_name = self._collect_db_table_name(components, app_name)  # Changed from app_label to components, app_name
        model_permission_path = self._collect_model_permission_path(app_path, model_name)

        permission_data = self._collect_permission_inheritance(components)

        verbose_name, verbose_name_plural = self._derive_verbose_names(model_name, model_name_plural)

        return {
            'app_path': app_path,
            'app_name': app_name,
            'model_name': model_name,
            'model_name_plural': model_name_plural,
            'app_label': app_label,
            'db_table_name': db_table_name,
            'model_permission_path': model_permission_path,
            'verbose_name': verbose_name,
            'verbose_name_plural': verbose_name_plural,
            **permission_data,
        }

    def _collect_app_label(self, components: list[str], app_name: str) -> str:
        """
        Prompts the user for the Django app label.

        :param components: List of app path components.
        :param app_name: Name of the app.
        :return: User-provided or default app label.
        """

        immediate_parent = components[-2] if len(components) > 2 else None
        default = immediate_parent.lower() + '_' + app_name.lower() if immediate_parent else app_name.lower()
        return self._collect_input('Enter the app label', default, '3/8')

    def _collect_app_name(self, components: list[str]) -> str:
        """
        Prompts the user for the app name.

        :param components: List of app path components.
        :return: User-provided or default app name.
        """

        default = components[-1]
        return self._collect_input('Enter the app name', default, '2/8')

    def _collect_app_path(self) -> str:
        """
        Prompts the user for the app path and validates it.

        :return: Validated app path in dot notation.
        :raises CommandError: If the app path is empty or invalid.
        """

        app_path = self._collect_simple_input('Enter the app path (e.g., "app.human_resource.employee.skill")', '1/8')

        if not app_path:
            self.reporter.write('\n', self.reporter.style_notice)

            message = 'The app path is required'
            raise CommandError(message)

        components = app_path.split('.')
        self.validator.validate_app_path(components)

        return app_path

    def _collect_db_table_name(self, components: list[str], app_name: str) -> str:
        """
        Prompts the user for the database table name.

        :param components: List of app path components.
        :param app_name: Name of the app.
        :return: User-provided or default database table name.
        """

        parent_parts = components[1:-1] if len(components) > 1 else []
        default = '_'.join(parent_parts).lower() + '_' + app_name.lower() if parent_parts else app_name.lower()
        return self._collect_input('Enter the database table name', default, '6/8')

    def _collect_input(self, prompt: str, default: str, step_number: str) -> str:
        """
        Prompts the user for input with a default value.

        :param prompt: Prompt message to display.
        :param default: Default value if user presses Enter.
        :param step_number: Step number in the wizard (e.g., '1/8').
        :return: User-provided input or default value.
        """

        self.reporter.write(f'\n[{step_number}]: {prompt} (default: "{default}")', self.reporter.style_notice)
        user_input = input('Press Enter to use default or type a custom value: ').strip()
        return user_input if user_input else default

    def _collect_model_name(self, app_name: str) -> str:
        """
        Prompts the user for the model class name.

        :param app_name: App name to derive default from.
        :return: User-provided or default model name in TitleCase.
        """

        default = ''.join(word.title() for word in app_name.split('_'))
        return self._collect_input('Enter the model name', default, '4/8')

    def _collect_model_name_plural(self, model_name: str) -> str:
        """
        Prompts the user for the plural form of the model name.

        :param model_name: Singular model name.
        :return: User-provided or default plural model name.
        """

        default = model_name + 's'
        return self._collect_input('Enter the model name plural', default, '5/8')

    def _collect_model_permission_path(self, app_path: str, model_name: str) -> str:
        """
        Prompts the user for the model permission path.

        :param app_path: Dot-separated app path.
        :param model_name: Model class name.
        :return: User-provided or default model permission path.
        """

        default = f'{app_path}.models.{model_name}'
        return self._collect_input('Enter the model permission path', default, '7/8')

    def _collect_permission_inheritance(self, components: list[str]) -> dict[str, str]:
        """
        Collects permission inheritance configuration if applicable.

        Prompts the user about inheriting permissions from parent apps
        and collects necessary parent model information.

        :param components: List of app path components.
        :return: Dictionary containing permission inheritance settings.
        """

        if len(components) <= 1:
            return {'inherit_permissions': False, 'parent_permission_prefix': '', 'parent_model_instance_name': ''}

        if not self._should_inherit_permissions():
            return {'inherit_permissions': False, 'parent_permission_prefix': '', 'parent_model_instance_name': ''}

        parent_parts = components[1:-1]
        parent_name = components[-2]
        parent_model_class = ''.join(word.title() for word in parent_name.split('_'))

        self.reporter.write('\n[Permission Inheritance Configuration]\n', self.reporter.style_notice)

        return {
            'inherit_permissions': True,
            'parent_permission_prefix': self._collect_input(
                'Enter the parent permission prefix',
                '_'.join(parent_parts).lower(),
                '1/3'
            ),
            'parent_model_instance_name': self._collect_input(
                'Enter the parent model instance name',
                parent_name.lower(),
                '2/3'
            ),
            'parent_model_path': self._collect_input(
                'Enter the parent model path',
                '.'.join(components[:-1]) + f'.models.{parent_model_class}',
                '3/3'
            ),
        }

    def _collect_simple_input(self, prompt: str, step_number: str) -> str:
        """
        Prompts the user for simple input without a default value.

        :param prompt: Prompt message to display.
        :param step_number: Step number in the wizard.
        :return: User-provided input.
        """

        return input(f'[{step_number}]: {prompt}: ').strip()

    def _derive_verbose_names(self, model_name: str, model_name_plural: str) -> tuple[str, str]:
        """
        Derives human-readable verbose names from model names.

        Converts CamelCase model names to space-separated words.

        :param model_name: Singular model name in CamelCase.
        :param model_name_plural: Plural model name in CamelCase.
        :return: Tuple of (verbose_name, verbose_name_plural).
        """

        verbose_name = re.sub(r'(?<!^)(?=[A-Z])', ' ', model_name)
        verbose_name_plural = re.sub(r'(?<!^)(?=[A-Z])', ' ', model_name_plural)
        return verbose_name, verbose_name_plural

    def _should_inherit_permissions(self) -> bool:
        """
        Prompts the user to confirm permission inheritance.

        :return: True if user wants to inherit permissions, False otherwise.
        """

        self.reporter.write('\n[8/8]: Do you want this app to inherit permissions from its parent? (y/n)', self.reporter.style_notice)
        user_input = input('Default is "n": ').strip().lower()
        return user_input == 'y'



from __future__ import annotations

from django.urls import reverse

from django_spire.core.tests.test_cases import BaseTestCase
from django_spire.knowledge.collection.tests.factories import create_test_collection


class CollectionJsonUrlsTests(BaseTestCase):
    def setUp(self):
        super().setUp()

        self.test_collection = create_test_collection()

    def test_reorder_view_url_path(self):
        response = self.client.post(
            reverse('django_spire:knowledge:collection:json:reorder'),
            data={
                'collection_id': self.test_collection.pk,
                'order': 0
            },
            content_type='application/json',
        )
        assert response.status_code == 200

    def test_reorder_view_with_parent(self):
        child_collection = create_test_collection(
            parent=self.test_collection,
            name='Child Collection'
        )
        response = self.client.post(
            reverse('django_spire:knowledge:collection:json:reorder'),
            data={
                'collection_id': child_collection.pk,
                'order': 0,
                'parent': self.test_collection.pk
            },
            content_type='application/json',
        )
        assert response.status_code == 200

    def test_reorder_view_invalid_collection(self):
        response = self.client.post(
            reverse('django_spire:knowledge:collection:json:reorder'),
            data={
                'collection_id': 99999,
                'order': 0
            },
            content_type='application/json',
        )
        assert response.status_code == 200
        assert response.json()['type'] == 'error'

    def test_reorder_view_missing_order(self):
        response = self.client.post(
            reverse('django_spire:knowledge:collection:json:reorder'),
            data={
                'collection_id': self.test_collection.pk,
            },
            content_type='application/json',
        )
        assert response.status_code == 200
        assert response.json()['type'] == 'error'



from __future__ import annotations

from typing import TYPE_CHECKING
from unittest.mock import Mock, patch

from dandy.llm.request.message import MessageHistory
from django.contrib.auth.models import Permission, User
from django.test import RequestFactory, override_settings

from django_spire.ai.chat.intelligence.decoders.intent_decoder import generate_intent_decoder
from django_spire.ai.chat.intelligence.workflows.chat_workflow import chat_workflow
from django_spire.ai.chat.message_intel import BaseMessageIntel, DefaultMessageIntel
from django_spire.ai.chat.router import BaseChatRouter
from django_spire.core.tests.test_cases import BaseTestCase

if TYPE_CHECKING:
    from django.core.handlers.wsgi import WSGIRequest


class KnowledgeRouter(BaseChatRouter):
    def workflow(
        self,
        request: WSGIRequest,
        user_input: str,
        message_history: MessageHistory | None = None
    ) -> DefaultMessageIntel:
        return DefaultMessageIntel(text='Knowledge search result')


class SupportRouter(BaseChatRouter):
    def workflow(
        self,
        request: WSGIRequest,
        user_input: str,
        message_history: MessageHistory | None = None
    ) -> DefaultMessageIntel:
        return DefaultMessageIntel(text='Support response')


class TestRouterIntegration(BaseTestCase):
    def setUp(self) -> None:
        super().setUp()

        self.factory = RequestFactory()
        self.request = self.factory.get('/')
        self.request.user = self.super_user

    @override_settings(
        DJANGO_SPIRE_AI_DEFAULT_CHAT_ROUTER='SPIRE',
        DJANGO_SPIRE_AI_CHAT_ROUTERS={
            'SPIRE': 'django_spire.ai.chat.router.SpireChatRouter',
            'KNOWLEDGE': 'django_spire.ai.chat.tests.test_router.test_integration.KnowledgeRouter',
            'SUPPORT': 'django_spire.ai.chat.tests.test_router.test_integration.SupportRouter',
        },
        DJANGO_SPIRE_AI_INTENT_CHAT_ROUTERS={
            'KNOWLEDGE_SEARCH': {
                'INTENT_DESCRIPTION': 'User asking about documentation',
                'REQUIRED_PERMISSION': 'auth.view_user',
                'CHAT_ROUTER': 'django_spire.ai.chat.tests.test_router.test_integration.KnowledgeRouter',
            },
            'SUPPORT': {
                'INTENT_DESCRIPTION': 'User needs support',
                'CHAT_ROUTER': 'django_spire.ai.chat.tests.test_router.test_integration.SupportRouter',
            }
        }
    )
    def test_intent_routing_with_permission(self) -> None:
        permission = Permission.objects.get(codename='view_user')
        self.super_user.user_permissions.add(permission)

        with patch('django_spire.ai.chat.router.generate_intent_decoder') as mock_decoder:
            mock_decoder_instance = type('MockDecoder', (), {
                'process': lambda self, user_input, **kwargs: [
                    lambda **kwargs: KnowledgeRouter().workflow(**kwargs)
                ]
            })()

            mock_decoder.return_value = mock_decoder_instance

            result = chat_workflow(
                request=self.request,
                user_input='Tell me about the documentation',
                message_history=None
            )

            assert isinstance(result, BaseMessageIntel)

    @override_settings(
        DJANGO_SPIRE_AI_DEFAULT_CHAT_ROUTER='SUPPORT',
        DJANGO_SPIRE_AI_CHAT_ROUTERS={
            'SUPPORT': 'django_spire.ai.chat.tests.test_router.test_integration.SupportRouter',
        }
    )
    def test_direct_router_selection(self) -> None:
        result = chat_workflow(
            request=self.request,
            user_input='I need help',
            message_history=None
        )

        assert isinstance(result, DefaultMessageIntel)
        assert result.text == 'Support response'

    @override_settings(
        DJANGO_SPIRE_AI_INTENT_CHAT_ROUTERS={
            'RESTRICTED': {
                'INTENT_DESCRIPTION': 'Restricted intent',
                'REQUIRED_PERMISSION': 'auth.delete_user',
                'CHAT_ROUTER': 'django_spire.ai.chat.tests.test_router.test_integration.SupportRouter',
            }
        }
    )
    def test_intent_excluded_without_permission(self) -> None:
        regular_user = User.objects.create_user(username='regular', password='test')

        request = self.factory.get('/')
        request.user = regular_user

        decoder = generate_intent_decoder(request=request, default_callable=None)

        assert 'Restricted intent' not in decoder.mapping

    @override_settings(
        DJANGO_SPIRE_AI_DEFAULT_CHAT_ROUTER='KNOWLEDGE',
        DJANGO_SPIRE_AI_CHAT_ROUTERS={
            'KNOWLEDGE': 'django_spire.ai.chat.tests.test_router.test_integration.KnowledgeRouter',
        }
    )
    def test_end_to_end_workflow(self) -> None:
        message_history = MessageHistory()
        message_history.add_message(role='user', content='Previous message')

        result = chat_workflow(
            request=self.request,
            user_input='Current message',
            message_history=message_history
        )

        assert isinstance(result, BaseMessageIntel)
        assert isinstance(result, DefaultMessageIntel)

    @override_settings(
        DJANGO_SPIRE_AI_CHAT_ROUTERS={}
    )
    def test_workflow_handles_empty_routers(self) -> None:
        with patch('django_spire.ai.chat.intelligence.workflows.chat_workflow.get_callable_from_module_string_and_validate_arguments') as mock_get_callable:
            mock_router_class = Mock()
            mock_router_instance = Mock()
            mock_router_instance.process.return_value = DefaultMessageIntel(text='Fallback')
            mock_router_class.return_value = mock_router_instance
            mock_get_callable.return_value = mock_router_class

            result = chat_workflow(
                request=self.request,
                user_input='Test',
                message_history=None
            )

            assert isinstance(result, DefaultMessageIntel)

    @override_settings(
        DJANGO_SPIRE_AI_DEFAULT_CHAT_ROUTER='KNOWLEDGE',
        DJANGO_SPIRE_AI_CHAT_ROUTERS={
            'KNOWLEDGE': 'django_spire.ai.chat.tests.test_router.test_integration.KnowledgeRouter',
            'SUPPORT': 'django_spire.ai.chat.tests.test_router.test_integration.SupportRouter',
        }
    )
    def test_router_selection_by_key(self) -> None:
        result = chat_workflow(
            request=self.request,
            user_input='Test',
            message_history=None
        )

        assert isinstance(result, DefaultMessageIntel)
        assert result.text == 'Knowledge search result'

    @override_settings(
        DJANGO_SPIRE_AI_DEFAULT_CHAT_ROUTER='SUPPORT',
        DJANGO_SPIRE_AI_CHAT_ROUTERS={
            'SUPPORT': 'django_spire.ai.chat.tests.test_router.test_integration.SupportRouter',
        }
    )
    def test_workflow_with_none_message_history(self) -> None:
        result = chat_workflow(
            request=self.request,
            user_input='Test input',
            message_history=None
        )

        assert isinstance(result, DefaultMessageIntel)
        assert result.text == 'Support response'

    @override_settings(
        DJANGO_SPIRE_AI_DEFAULT_CHAT_ROUTER='SUPPORT',
        DJANGO_SPIRE_AI_CHAT_ROUTERS={
            'SUPPORT': 'django_spire.ai.chat.tests.test_router.test_integration.SupportRouter',
        }
    )
    def test_workflow_preserves_user_input(self) -> None:
        test_input = 'This is a specific test input'

        with patch.object(SupportRouter, 'workflow', wraps=SupportRouter().workflow) as mock_workflow:
            mock_workflow.return_value = DefaultMessageIntel(text='Response')

            chat_workflow(
                request=self.request,
                user_input=test_input,
                message_history=None
            )

    @override_settings(
        DJANGO_SPIRE_AI_DEFAULT_CHAT_ROUTER='KNOWLEDGE',
        DJANGO_SPIRE_AI_CHAT_ROUTERS={
            'KNOWLEDGE': 'django_spire.ai.chat.tests.test_router.test_integration.KnowledgeRouter',
        },
        DJANGO_SPIRE_AI_INTENT_CHAT_ROUTERS={}
    )
    def test_workflow_without_intent_routers(self) -> None:
        result = chat_workflow(
            request=self.request,
            user_input='Test',
            message_history=None
        )

        assert isinstance(result, DefaultMessageIntel)
        assert result.text == 'Knowledge search result'



from __future__ import annotations

import pytest

from django_spire.auth.user.tests.factories import create_user
from django_spire.core.tests.test_cases import BaseTestCase
from django_spire.notification.app.exceptions import AppNotificationError
from django_spire.notification.app.models import AppNotification
from django_spire.notification.app.processor import AppNotificationProcessor
from django_spire.notification.choices import (
    NotificationStatusChoices,
    NotificationTypeChoices,
)
from django_spire.notification.exceptions import NotificationError
from django_spire.notification.models import Notification


class AppNotificationProcessorTests(BaseTestCase):
    def setUp(self):
        super().setUp()
        self.user = create_user(username='test_processor_user')
        self.processor = AppNotificationProcessor()

    def _create_notification(self, **kwargs) -> Notification:
        data = {
            'user': self.user,
            'type': NotificationTypeChoices.APP,
            'title': 'Test Notification',
            'body': 'Test body',
            'status': NotificationStatusChoices.PENDING,
        }
        data.update(kwargs)
        notification = Notification.objects.create(**data)
        AppNotification.objects.create(notification=notification)
        return notification

    def test_process_sets_status_to_sent(self):
        notification = self._create_notification()

        self.processor.process(notification)

        notification.refresh_from_db()
        assert notification.status == NotificationStatusChoices.SENT

    def test_process_sets_sent_datetime(self):
        notification = self._create_notification()

        self.processor.process(notification)

        notification.refresh_from_db()
        assert notification.sent_datetime is not None

    def test_process_raises_error_for_wrong_type(self):
        notification = self._create_notification(type=NotificationTypeChoices.EMAIL)

        with pytest.raises(NotificationError):
            self.processor.process(notification)

    def test_process_raises_error_for_missing_user(self):
        notification = Notification.objects.create(
            user=None,
            type=NotificationTypeChoices.APP,
            title='Test',
            body='Test',
            status=NotificationStatusChoices.PENDING,
        )
        AppNotification.objects.create(notification=notification)

        with pytest.raises(AppNotificationError):
            self.processor.process(notification)

    def test_process_list(self):
        notifications = [self._create_notification() for _ in range(3)]

        self.processor.process_list(notifications)

        for notification in notifications:
            notification.refresh_from_db()
            assert notification.status == NotificationStatusChoices.SENT
            assert notification.sent_datetime is not None

    def test_process_list_raises_error_for_wrong_type(self):
        notifications = [
            self._create_notification(),
            self._create_notification(type=NotificationTypeChoices.EMAIL),
        ]

        with pytest.raises(NotificationError):
            self.processor.process_list(notifications)

    def test_process_ready(self):
        pending_notification = self._create_notification()

        sent_notification = self._create_notification(
            status=NotificationStatusChoices.SENT
        )

        self.processor.process_ready()

        pending_notification.refresh_from_db()
        assert pending_notification.status == NotificationStatusChoices.SENT

    def test_process_errored(self):
        errored_notification = self._create_notification(
            status=NotificationStatusChoices.ERRORED
        )

        self.processor.process_errored()

        errored_notification.refresh_from_db()
        assert errored_notification.status == NotificationStatusChoices.SENT



from __future__ import annotations

import json

from django.urls import reverse

from django_spire.ai.chat.tests.test_urls.factories import create_test_chat
from django_spire.core.tests.test_cases import BaseTestCase


class ChatJsonUrlTests(BaseTestCase):
    def setUp(self) -> None:
        super().setUp()

        self.test_chat = create_test_chat(user=self.super_user)

    def test_delete_view_url_path(self) -> None:
        response = self.client.post(
            reverse(
                'django_spire:ai:chat:json:delete',
                kwargs={'pk': self.test_chat.pk}
            ),
            content_type='application/json'
        )

        assert response.status_code == 200

    def test_delete_view_success_response(self) -> None:
        response = self.client.post(
            reverse(
                'django_spire:ai:chat:json:delete',
                kwargs={'pk': self.test_chat.pk}
            ),
            content_type='application/json'
        )

        data = response.json()

        assert data['type'] == 'success'
        assert 'deleted' in data['message'].lower()

    def test_delete_view_sets_deleted_flag(self) -> None:
        self.client.post(
            reverse(
                'django_spire:ai:chat:json:delete',
                kwargs={'pk': self.test_chat.pk}
            ),
            content_type='application/json'
        )

        self.test_chat.refresh_from_db()

        assert self.test_chat.is_deleted

    def test_delete_view_nonexistent_chat(self) -> None:
        response = self.client.post(
            reverse(
                'django_spire:ai:chat:json:delete',
                kwargs={'pk': 99999}
            ),
            content_type='application/json'
        )

        data = response.json()

        assert data['type'] == 'error'

    def test_rename_view_url_path(self) -> None:
        response = self.client.post(
            reverse(
                'django_spire:ai:chat:json:rename',
                kwargs={'pk': self.test_chat.pk}
            ),
            data=json.dumps({'new_name': 'New Chat Name'}),
            content_type='application/json'
        )

        assert response.status_code == 200

    def test_rename_view_success_response(self) -> None:
        response = self.client.post(
            reverse(
                'django_spire:ai:chat:json:rename',
                kwargs={'pk': self.test_chat.pk}
            ),
            data=json.dumps({'new_name': 'Updated Name'}),
            content_type='application/json'
        )

        data = response.json()

        assert data['type'] == 'success'

    def test_rename_view_updates_name(self) -> None:
        new_name = 'Brand New Name'

        self.client.post(
            reverse(
                'django_spire:ai:chat:json:rename',
                kwargs={'pk': self.test_chat.pk}
            ),
            data=json.dumps({'new_name': new_name}),
            content_type='application/json'
        )

        self.test_chat.refresh_from_db()

        assert self.test_chat.name == new_name

    def test_rename_view_empty_name(self) -> None:
        response = self.client.post(
            reverse(
                'django_spire:ai:chat:json:rename',
                kwargs={'pk': self.test_chat.pk}
            ),
            data=json.dumps({'new_name': ''}),
            content_type='application/json'
        )

        data = response.json()

        assert data['type'] == 'error'

    def test_rename_view_name_too_long(self) -> None:
        long_name = 'A' * 200

        response = self.client.post(
            reverse(
                'django_spire:ai:chat:json:rename',
                kwargs={'pk': self.test_chat.pk}
            ),
            data=json.dumps({'new_name': long_name}),
            content_type='application/json'
        )

        data = response.json()

        assert data['type'] == 'error'

    def test_rename_view_nonexistent_chat(self) -> None:
        response = self.client.post(
            reverse(
                'django_spire:ai:chat:json:rename',
                kwargs={'pk': 99999}
            ),
            data=json.dumps({'new_name': 'New Name'}),
            content_type='application/json'
        )

        data = response.json()

        assert data['type'] == 'error'

    def test_rename_view_max_length_name(self) -> None:
        max_length_name = 'A' * 128

        response = self.client.post(
            reverse(
                'django_spire:ai:chat:json:rename',
                kwargs={'pk': self.test_chat.pk}
            ),
            data=json.dumps({'new_name': max_length_name}),
            content_type='application/json'
        )

        data = response.json()

        assert data['type'] == 'success'

        self.test_chat.refresh_from_db()
        assert self.test_chat.name == max_length_name


class ChatJsonUrlAdditionalTests(BaseTestCase):
    def test_delete_multiple_chats(self) -> None:
        chat1 = create_test_chat(user=self.super_user)
        chat2 = create_test_chat(user=self.super_user)

        self.client.post(
            reverse('django_spire:ai:chat:json:delete', kwargs={'pk': chat1.pk}),
            content_type='application/json'
        )

        self.client.post(
            reverse('django_spire:ai:chat:json:delete', kwargs={'pk': chat2.pk}),
            content_type='application/json'
        )

        chat1.refresh_from_db()
        chat2.refresh_from_db()

        assert chat1.is_deleted
        assert chat2.is_deleted

    def test_rename_with_special_characters(self) -> None:
        chat = create_test_chat(user=self.super_user)
        special_name = "Chat with 'quotes' and \"double quotes\""

        response = self.client.post(
            reverse('django_spire:ai:chat:json:rename', kwargs={'pk': chat.pk}),
            data=json.dumps({'new_name': special_name}),
            content_type='application/json'
        )

        data = response.json()

        assert data['type'] == 'success'

    def test_rename_with_unicode(self) -> None:
        chat = create_test_chat(user=self.super_user)
        unicode_name = 'Chat 日本語 🎉'

        response = self.client.post(
            reverse('django_spire:ai:chat:json:rename', kwargs={'pk': chat.pk}),
            data=json.dumps({'new_name': unicode_name}),
            content_type='application/json'
        )

        data = response.json()

        assert data['type'] == 'success'

        chat.refresh_from_db()
        assert chat.name == unicode_name



from __future__ import annotations

from typing import TYPE_CHECKING

if TYPE_CHECKING:
    from playwright.sync_api import Locator


class Dropdown:
    """Playwright component for django_spire/dropdown/dropdown.html"""

    menu_selector: str = '.position-absolute.shadow-lg.card'
    trigger_selector: str = '[x-bind="trigger"]'

    def __init__(self, parent_locator: Locator) -> None:
        self.parent = parent_locator

    @property
    def menu(self) -> Locator:
        return self.parent.page.locator(self.menu_selector).filter(
            has=self.parent.page.locator(':visible')
        ).first

    @property
    def trigger(self) -> Locator:
        return self.parent.locator(self.trigger_selector)

    def click_option(self, text: str) -> None:
        self.menu.locator(f'text={text}').click()

    def close(self) -> None:
        if self.is_open():
            self.trigger.click()

    def get_option(self, text: str) -> Locator:
        return self.menu.locator(f'text={text}')

    def has_option(self, text: str) -> bool:
        return self.get_option(text).is_visible()

    def is_open(self) -> bool:
        return self.menu.is_visible()

    def open(self) -> None:
        if not self.is_open():
            self.trigger.click()


class EllipsisDropdown(Dropdown):
    """Playwright component for django_spire/dropdown/ellipsis_dropdown.html"""

    trigger_selector: str = '.bi-three-dots-vertical'

    def click_delete(self) -> None:
        self.click_option('Delete')

    def click_edit(self) -> None:
        self.click_option('Edit')

    def click_view(self) -> None:
        self.click_option('View')

    def has_delete_option(self) -> bool:
        return self.has_option('Delete')

    def has_edit_option(self) -> bool:
        return self.has_option('Edit')

    def has_view_option(self) -> bool:
        return self.has_option('View')


class EllipsisModalDropdown(EllipsisDropdown):
    """
    Playwright component for django_spire/dropdown/ellipsis_modal_dropdown.html
    Dropdown options trigger modals via dispatch_modal_view()
    """

    pass


class EllipsisTableDropdown(EllipsisDropdown):
    """
    Playwright component for django_spire/dropdown/ellipsis_table_dropdown.html
    Used in table rows, positioned start-0 instead of end-0
    """
    trigger_selector: str = 'td .bi-three-dots-vertical'



from __future__ import annotations

from django_spire.knowledge.collection import models

from django_spire.contrib.seeding import DjangoModelSeeder


class CollectionSeeder(DjangoModelSeeder):
    model_class = models.Collection
    # cache_name = 'collection_seeder'
    fields = {
        'id': 'exclude',
        'parent': 'exclude',
        'name': (
            'llm',
            'A name for a collection of documents. Make it fun and give it a theme'
        ),
        'description': (
            'llm',
            'Short description on the what the documents are about, keep it related to the name.'
        ),
    }

    @classmethod
    def _correct_order(
        cls,
        child_collections: list[models.Collection]
    ) -> list[models.Collection]:
        parent_collections = cls.model_class.objects.parentless()

        for parent_collection in parent_collections:
            children = parent_collection.children.all()
            for idx, child_collection in enumerate(children):
                child_collection.order = idx

            cls.model_class.objects.bulk_update(children, ['order'])

        return child_collections

    @classmethod
    def seed_child_collections(cls, count: int = 1) -> list[models.Collection]:
        return cls._correct_order(
            cls.seed_database(
                count=count,
                fields=cls.fields | {
                    'parent_id': ('custom', 'fk_random', {'model_class': models.Collection})
                },
            )
        )



from __future__ import annotations

from django.conf import settings
from django.test import override_settings

from django_spire.core.tests.test_cases import BaseTestCase
from django_spire.file.interfaces import (
    FileFormatter,
    MultiFileUploader,
    SingleFileUploader,
)
from django_spire.file.models import File
from django_spire.file.tests.factories import create_test_in_memory_uploaded_file


STORAGES_OVERRIDE = {
    'default': {
        'BACKEND': 'django.core.files.storage.FileSystemStorage',
    },
    'staticfiles': {
        'BACKEND': 'django.contrib.staticfiles.storage.StaticFilesStorage',
    },
}


class FileFormatterTests(BaseTestCase):
    def setUp(self):
        super().setUp()

        self.uploaded_file = create_test_in_memory_uploaded_file(
            name='document',
            file_type='pdf',
            content=b'test content',
        )
        self.formatter = FileFormatter(file=self.uploaded_file)

    def test_format_name(self):
        assert self.formatter.name == 'document'

    def test_parse_type(self):
        assert self.formatter.type == 'pdf'

    def test_location_contains_base_folder(self):
        assert self.formatter.location.startswith(settings.BASE_FOLDER_NAME + '/')

    def test_location_contains_app_name(self):
        assert '/Uncategorized/' in self.formatter.location

    def test_location_contains_file_name(self):
        assert self.formatter.location.endswith('document')

    def test_location_with_related_field(self):
        formatter = FileFormatter(
            file=self.uploaded_file,
            related_field='abc',
        )

        assert '/abc/' in formatter.location

    def test_location_with_custom_app_name(self):
        formatter = FileFormatter(
            file=self.uploaded_file,
            app_name='CustomApp',
        )

        assert '/CustomApp/' in formatter.location

    def test_size_verbose_kb(self):
        content = b'x' * 1000
        uploaded_file = create_test_in_memory_uploaded_file(content=content)
        formatter = FileFormatter(file=uploaded_file)

        assert 'kb' in formatter.size_verbose()

    def test_size_verbose_mb(self):
        content = b'x' * 1000000
        uploaded_file = create_test_in_memory_uploaded_file(content=content)
        formatter = FileFormatter(file=uploaded_file)

        assert 'Mb' in formatter.size_verbose()

    @override_settings(STORAGES=STORAGES_OVERRIDE)
    def test_null_file_obj_returns_file(self):
        result = self.formatter.null_file_obj()

        assert isinstance(result, File)
        assert result.name == 'document'
        assert result.type == 'pdf'

    @override_settings(STORAGES=STORAGES_OVERRIDE)
    def test_null_file_obj_not_saved(self):
        result = self.formatter.null_file_obj()

        assert result.pk is None


@override_settings(STORAGES=STORAGES_OVERRIDE)
class SingleFileUploaderTests(BaseTestCase):
    def setUp(self):
        super().setUp()

        self.uploader = SingleFileUploader(related_field='abc')
        self.uploaded_file = create_test_in_memory_uploaded_file()

    def test_upload_creates_file(self):
        initial_count = File.objects.count()

        self.uploader.upload(self.uploaded_file)

        assert File.objects.count() == initial_count + 1

    def test_upload_returns_file(self):
        result = self.uploader.upload(self.uploaded_file)

        assert isinstance(result, File)
        assert result.pk is not None

    def test_upload_sets_name(self):
        result = self.uploader.upload(self.uploaded_file)

        assert result.name == 'test_file'

    def test_upload_sets_type(self):
        result = self.uploader.upload(self.uploaded_file)

        assert result.type == 'pdf'

    def test_null_file_obj(self):
        result = self.uploader.null_file_obj(self.uploaded_file)

        assert isinstance(result, File)
        assert result.pk is None


@override_settings(STORAGES=STORAGES_OVERRIDE)
class MultiFileUploaderTests(BaseTestCase):
    def setUp(self):
        super().setUp()

        self.uploader = MultiFileUploader(related_field='abc')
        self.uploaded_files = [
            create_test_in_memory_uploaded_file(name='file1'),
            create_test_in_memory_uploaded_file(name='file2'),
        ]

    def test_upload_creates_multiple_files(self):
        initial_count = File.objects.count()

        self.uploader.upload(self.uploaded_files)

        assert File.objects.count() == initial_count + 2

    def test_upload_returns_list_of_files(self):
        result = self.uploader.upload(self.uploaded_files)

        assert isinstance(result, list)
        assert len(result) == 2

    def test_upload_returns_file_instances(self):
        result = self.uploader.upload(self.uploaded_files)

        for file in result:
            assert isinstance(file, File)
            assert file.pk is not None

    def test_upload_sets_names(self):
        result = self.uploader.upload(self.uploaded_files)
        names = [f.name for f in result]

        assert 'file1' in names
        assert 'file2' in names

    def test_upload_empty_list(self):
        result = self.uploader.upload([])

        assert result == []

    def test_null_file_obj(self):
        uploaded_file = create_test_in_memory_uploaded_file()
        result = self.uploader.null_file_obj(uploaded_file)

        assert isinstance(result, File)
        assert result.pk is None



from __future__ import annotations

from django.test import TestCase

from django_spire.contrib.utils import truncate_string


class TestTruncateString(TestCase):
    def test_returns_original_when_equal_to_length(self) -> None:
        result = truncate_string('hello', 5)

        assert result == 'hello'

    def test_returns_original_when_shorter_than_length(self) -> None:
        result = truncate_string('hi', 10)

        assert result == 'hi'

    def test_truncates_when_longer_than_length(self) -> None:
        result = truncate_string('hello world', 8)

        assert result == 'hello...'

    def test_truncates_with_ellipsis(self) -> None:
        result = truncate_string('this is a long string', 10)

        assert result.endswith('...')

    def test_truncated_length_equals_specified_length(self) -> None:
        result = truncate_string('this is a long string', 10)

        assert len(result) == 10

    def test_empty_string(self) -> None:
        result = truncate_string('', 5)

        assert result == ''



from __future__ import annotations

from django_spire.core.tests.test_cases import BaseTestCase
from django_spire.notification.app.models import AppNotification
from django_spire.notification.choices import NotificationTypeChoices
from django_spire.notification.email.models import EmailNotification
from django_spire.notification.maps import NOTIFICATION_TYPE_CHOICE_TO_MODEL_MAP
from django_spire.notification.push.models import PushNotification
from django_spire.notification.sms.models import SmsNotification


class NotificationMapsTests(BaseTestCase):
    def test_map_contains_app(self):
        assert NotificationTypeChoices.APP in NOTIFICATION_TYPE_CHOICE_TO_MODEL_MAP
        assert NOTIFICATION_TYPE_CHOICE_TO_MODEL_MAP[NotificationTypeChoices.APP] == AppNotification

    def test_map_contains_email(self):
        assert NotificationTypeChoices.EMAIL in NOTIFICATION_TYPE_CHOICE_TO_MODEL_MAP
        assert NOTIFICATION_TYPE_CHOICE_TO_MODEL_MAP[NotificationTypeChoices.EMAIL] == EmailNotification

    def test_map_contains_push(self):
        assert NotificationTypeChoices.PUSH in NOTIFICATION_TYPE_CHOICE_TO_MODEL_MAP
        assert NOTIFICATION_TYPE_CHOICE_TO_MODEL_MAP[NotificationTypeChoices.PUSH] == PushNotification

    def test_map_contains_sms(self):
        assert NotificationTypeChoices.SMS in NOTIFICATION_TYPE_CHOICE_TO_MODEL_MAP
        assert NOTIFICATION_TYPE_CHOICE_TO_MODEL_MAP[NotificationTypeChoices.SMS] == SmsNotification

    def test_map_has_all_notification_types(self):
        for choice in NotificationTypeChoices:
            assert choice in NOTIFICATION_TYPE_CHOICE_TO_MODEL_MAP



from __future__ import annotations

from django_spire.core.tests.test_cases import BaseTestCase
from django_spire.knowledge.entry.version.models import EntryVersion
from django_spire.knowledge.entry.version.tests.factories import create_test_entry_version
from django_spire.knowledge.entry.version.block.tests.factories import create_test_version_block


class EntryVersionQuerySetTests(BaseTestCase):
    def setUp(self):
        super().setUp()
        self.entry_version = create_test_entry_version()

    def test_prefetch_blocks(self):
        create_test_version_block(version=self.entry_version)
        result = EntryVersion.objects.prefetch_blocks().get(pk=self.entry_version.pk)
        assert result.blocks.count() == 1

    def test_prefetch_blocks_ordering(self):
        create_test_version_block(version=self.entry_version, order=2)
        create_test_version_block(version=self.entry_version, order=1)

        result = EntryVersion.objects.prefetch_blocks().get(pk=self.entry_version.pk)
        blocks = list(result.blocks.all())

        assert blocks[0].order < blocks[1].order



from __future__ import annotations

from typing import TYPE_CHECKING, Callable

# from django.conf import settings
# from django.template.response import TemplateResponse

# from django_spire.consts import MAINTENANCE_MODE_SETTINGS_NAME

if TYPE_CHECKING:
    from django.core.handlers.wsgi import WSGIRequest
    from django.http import HttpResponse


class MaintenanceMiddleware:
    def __init__(self, get_response: Callable[[WSGIRequest], HttpResponse]) -> None:
        self.get_response = get_response

    def __call__(self, request: WSGIRequest) -> HttpResponse:
        # maintenance_mode = getattr(settings, MAINTENANCE_MODE_SETTINGS_NAME)
        #
        # if maintenance_mode is None:
        #     raise ValueError(f'"{MAINTENANCE_MODE_SETTINGS_NAME}" must be set in the django settings.')
        #
        # if maintenance_mode:
        #     return TemplateResponse(
        #         request=request,
        #         template='django_spire/page/maintenance_page.html',
        #     )
        #

        return self.get_response(request)



from __future__ import annotations

import pytest

from typing import TYPE_CHECKING

from django.core.exceptions import PermissionDenied
from django.test import RequestFactory

from django_spire.auth.permissions.decorators import (
    permission_required,
    permission_required_decorator_function,
)
from django_spire.auth.user.tests.factories import create_super_user, create_user
from django_spire.core.tests.test_cases import BaseTestCase

if TYPE_CHECKING:
    from django.core.handlers.wsgi import WSGIRequest


class PermissionRequiredDecoratorFunctionTestCase(BaseTestCase):
    def setUp(self) -> None:
        super().setUp()

        self.factory = RequestFactory()
        self.user = create_user(username='testuser')
        self.user.set_password('password')
        self.user.save()

    def test_unauthenticated_user_redirects(self) -> None:
        request = self.factory.get('/test/')
        request.user = type('AnonymousUser', (), {'is_authenticated': False})()

        def dummy_view(_request: WSGIRequest) -> str:
            return 'success'

        response = permission_required_decorator_function(
            'some.permission',
            dummy_view,
            request
        )
        assert response.status_code == 302

    def test_user_without_permission_raises_denied(self) -> None:
        request = self.factory.get('/test/')
        request.user = self.user

        def dummy_view(_request: WSGIRequest) -> str:
            return 'success'

        with pytest.raises(PermissionDenied):
            permission_required_decorator_function(
                'some.nonexistent_permission',
                dummy_view,
                request
            )

    def test_superuser_has_all_permissions(self) -> None:
        superuser = create_super_user()
        request = self.factory.get('/test/')
        request.user = superuser

        def dummy_view(_request: WSGIRequest) -> str:
            return 'success'

        result = permission_required_decorator_function(
            'any.permission',
            dummy_view,
            request
        )
        assert result == 'success'

    def test_redirects_to_login(self) -> None:
        request = self.factory.get('/test/')
        request.user = type('AnonymousUser', (), {'is_authenticated': False})()

        def dummy_view(_request: WSGIRequest) -> str:
            return 'success'

        response = permission_required_decorator_function(
            'some.permission',
            dummy_view,
            request
        )
        assert 'login' in response.url.lower()

    def test_multiple_permissions_all_required(self) -> None:
        request = self.factory.get('/test/')
        request.user = self.user

        def dummy_view(_request: WSGIRequest) -> str:
            return 'success'

        with pytest.raises(PermissionDenied):
            permission_required_decorator_function(
                ['perm1', 'perm2'],
                dummy_view,
                request,
                all_required=True
            )

    def test_multiple_permissions_any_required(self) -> None:
        superuser = create_super_user()
        request = self.factory.get('/test/')
        request.user = superuser

        def dummy_view(_request: WSGIRequest) -> str:
            return 'success'

        result = permission_required_decorator_function(
            ['perm1', 'perm2'],
            dummy_view,
            request,
            all_required=False
        )
        assert result == 'success'

    def test_passes_args_to_view(self) -> None:
        superuser = create_super_user()
        request = self.factory.get('/test/')
        request.user = superuser

        def dummy_view(_request: WSGIRequest, arg1: str, arg2: str) -> str:
            return f'{arg1}-{arg2}'

        result = permission_required_decorator_function(
            'any.permission',
            dummy_view,
            request,
            'hello',
            'world'
        )
        assert result == 'hello-world'

    def test_passes_kwargs_to_view(self) -> None:
        superuser = create_super_user()
        request = self.factory.get('/test/')
        request.user = superuser

        def dummy_view(_request: WSGIRequest, **kwargs) -> str:
            return kwargs.get('key', 'default')

        result = permission_required_decorator_function(
            'any.permission',
            dummy_view,
            request,
            key='value'
        )
        assert result == 'value'

    def test_single_permission_as_string(self) -> None:
        superuser = create_super_user()
        request = self.factory.get('/test/')
        request.user = superuser

        def dummy_view(_request: WSGIRequest) -> str:
            return 'success'

        result = permission_required_decorator_function(
            'single.permission',
            dummy_view,
            request
        )
        assert result == 'success'

    def test_empty_permissions_list(self) -> None:
        request = self.factory.get('/test/')
        request.user = self.user

        def dummy_view(_request: WSGIRequest) -> str:
            return 'success'

        result = permission_required_decorator_function(
            [],
            dummy_view,
            request
        )
        assert result == 'success'


class PermissionRequiredDecoratorTestCase(BaseTestCase):
    def setUp(self) -> None:
        super().setUp()

        self.factory = RequestFactory()
        self.user = create_user(username='testuser')
        self.user.set_password('password')
        self.user.save()

    def test_decorator_without_permission(self) -> None:
        @permission_required('django_spire_auth_group.delete_authgroup')
        def dummy_view(_request: WSGIRequest) -> str:
            return 'success'

        request = self.factory.get('/test/')
        request.user = self.user

        with pytest.raises(PermissionDenied):
            dummy_view(request)

    def test_decorator_superuser_succeeds(self) -> None:
        superuser = create_super_user()

        @permission_required('any.permission')
        def dummy_view(_request: WSGIRequest) -> str:
            return 'success'

        request = self.factory.get('/test/')
        request.user = superuser
        result = dummy_view(request)
        assert result == 'success'

    def test_decorator_multiple_permissions_all_required_fails(self) -> None:
        @permission_required(
            'django_spire_auth_group.view_authgroup',
            'django_spire_auth_group.change_authgroup',
            all_required=True
        )
        def dummy_view(_request: WSGIRequest) -> str:
            return 'success'

        request = self.factory.get('/test/')
        request.user = self.user

        with pytest.raises(PermissionDenied):
            dummy_view(request)

    def test_decorator_preserves_function_name(self) -> None:
        @permission_required('any.permission')
        def my_special_view(_request: WSGIRequest) -> str:
            return 'success'

        assert my_special_view.__name__ == 'my_special_view'

    def test_decorator_with_args(self) -> None:
        superuser = create_super_user()

        @permission_required('any.permission')
        def dummy_view(_request: WSGIRequest, arg1: str, arg2: str) -> str:
            return f'{arg1}-{arg2}'

        request = self.factory.get('/test/')
        request.user = superuser
        result = dummy_view(request, 'hello', 'world')
        assert result == 'hello-world'

    def test_decorator_with_kwargs(self) -> None:
        superuser = create_super_user()

        @permission_required('any.permission')
        def dummy_view(_request: WSGIRequest, **kwargs) -> str:
            return kwargs.get('key', 'default')

        request = self.factory.get('/test/')
        request.user = superuser
        result = dummy_view(request, key='value')
        assert result == 'value'

    def test_decorator_multiple_permissions_any_required_superuser(self) -> None:
        superuser = create_super_user()

        @permission_required('perm1', 'perm2', all_required=False)
        def dummy_view(_request: WSGIRequest) -> str:
            return 'success'

        request = self.factory.get('/test/')
        request.user = superuser
        result = dummy_view(request)
        assert result == 'success'

    def test_decorator_empty_permissions(self) -> None:
        @permission_required()
        def dummy_view(_request: WSGIRequest) -> str:
            return 'success'

        request = self.factory.get('/test/')
        request.user = self.user
        result = dummy_view(request)
        assert result == 'success'

    def test_decorator_unauthenticated_redirects(self) -> None:
        @permission_required('any.permission')
        def dummy_view(_request: WSGIRequest) -> str:
            return 'success'

        request = self.factory.get('/test/')
        request.user = type('AnonymousUser', (), {'is_authenticated': False})()

        response = dummy_view(request)
        assert response.status_code == 302

    def test_decorator_view_returns_none(self) -> None:
        superuser = create_super_user()

        @permission_required('any.permission')
        def dummy_view(_request: WSGIRequest) -> None:
            return None

        request = self.factory.get('/test/')
        request.user = superuser
        result = dummy_view(request)
        assert result is None

    def test_decorator_view_returns_dict(self) -> None:
        superuser = create_super_user()

        @permission_required('any.permission')
        def dummy_view(_request: WSGIRequest) -> dict:
            return {'key': 'value'}

        request = self.factory.get('/test/')
        request.user = superuser
        result = dummy_view(request)
        assert result == {'key': 'value'}

    def test_decorator_single_permission_denied(self) -> None:
        @permission_required('nonexistent.permission')
        def dummy_view(_request: WSGIRequest) -> str:
            return 'success'

        request = self.factory.get('/test/')
        request.user = self.user

        with pytest.raises(PermissionDenied):
            dummy_view(request)

    def test_decorator_callable_preserves_docstring(self) -> None:
        @permission_required('any.permission')
        def dummy_view(_request: WSGIRequest) -> str:
            """A docstring."""
            return 'success'

        assert dummy_view.__doc__ == 'A docstring.'



from __future__ import annotations

import pytest

from unittest.mock import MagicMock

from django.test import TestCase

from django_spire.contrib.ordering.exceptions import OrderingMixinError, OrderingMixinGroupError
from django_spire.contrib.ordering.mixins import OrderingModelMixin
from django_spire.contrib.ordering.services.processor_service import OrderingProcessorService
from django_spire.contrib.ordering.services.service import OrderingService
from django_spire.contrib.ordering.validators import OrderingMixinValidator


class TestOrderingMixinError(TestCase):
    def test_can_be_raised(self) -> None:
        with pytest.raises(OrderingMixinError):
            message = 'Test'
            raise OrderingMixinError(message)

    def test_is_exception(self) -> None:
        assert issubclass(OrderingMixinError, Exception)

    def test_message(self) -> None:
        error = OrderingMixinError('Test error message')

        assert str(error) == 'Test error message'


class TestOrderingMixinGroupError(TestCase):
    def test_can_be_raised(self) -> None:
        with pytest.raises(OrderingMixinGroupError):
            message = 'Test'
            raise OrderingMixinGroupError(message)

    def test_is_exception(self) -> None:
        assert issubclass(OrderingMixinGroupError, Exception)

    def test_message(self) -> None:
        error = OrderingMixinGroupError('Test group error message')

        assert str(error) == 'Test group error message'


class TestOrderingMixinValidator(TestCase):
    def setUp(self) -> None:
        self.obj = MagicMock()
        self.obj.pk = 1

        self.destination_objects = MagicMock()
        self.destination_objects.__len__ = MagicMock(return_value=5)

        self.origin_objects = MagicMock()
        self.origin_objects.__len__ = MagicMock(return_value=5)

    def test_errors_contain_ordering_mixin_error(self) -> None:
        validator = OrderingMixinValidator(
            destination_objects=self.destination_objects,
            position=-1,
            obj=self.obj,
            origin_objects=self.origin_objects
        )

        validator.validate()

        assert all(isinstance(error, OrderingMixinError) for error in validator.errors)

    def test_errors_property_returns_list(self) -> None:
        validator = OrderingMixinValidator(
            destination_objects=self.destination_objects,
            position=0,
            obj=self.obj,
            origin_objects=self.origin_objects
        )

        assert isinstance(validator.errors, list)

    def test_validate_position_at_boundary(self) -> None:
        validator = OrderingMixinValidator(
            destination_objects=self.destination_objects,
            position=5,
            obj=self.obj,
            origin_objects=self.origin_objects
        )

        assert validator.validate() is True

    def test_validate_position_zero(self) -> None:
        validator = OrderingMixinValidator(
            destination_objects=self.destination_objects,
            position=0,
            obj=self.obj,
            origin_objects=self.origin_objects
        )

        assert validator.validate() is True

    def test_validate_returns_false_for_negative_position(self) -> None:
        validator = OrderingMixinValidator(
            destination_objects=self.destination_objects,
            position=-1,
            obj=self.obj,
            origin_objects=self.origin_objects
        )

        assert validator.validate() is False
        assert len(validator.errors) == 1

    def test_validate_returns_false_for_position_greater_than_objects(self) -> None:
        validator = OrderingMixinValidator(
            destination_objects=self.destination_objects,
            position=10,
            obj=self.obj,
            origin_objects=self.origin_objects
        )

        assert validator.validate() is False
        assert len(validator.errors) == 1

    def test_validate_returns_true_for_valid_position(self) -> None:
        validator = OrderingMixinValidator(
            destination_objects=self.destination_objects,
            position=2,
            obj=self.obj,
            origin_objects=self.origin_objects
        )

        assert validator.validate() is True


class TestOrderingModelMixin(TestCase):
    def test_has_order_field(self) -> None:
        assert hasattr(OrderingModelMixin, 'order')

    def test_has_ordering_services_in_dict(self) -> None:
        assert 'ordering_services' in OrderingModelMixin.__dict__

    def test_is_abstract(self) -> None:
        assert OrderingModelMixin._meta.abstract is True

    def test_order_default_value(self) -> None:
        assert OrderingModelMixin._meta.get_field('order').default == 0


class TestOrderingProcessorService(TestCase):
    def test_has_move_to_position_method(self) -> None:
        assert hasattr(OrderingProcessorService, 'move_to_position')

    def test_has_remove_from_objects_method(self) -> None:
        assert hasattr(OrderingProcessorService, 'remove_from_objects')

    def test_has_reorder_destination_and_origin_objects_method(self) -> None:
        assert hasattr(OrderingProcessorService, '_reorder_destination_and_origin_objects')

    def test_has_reorder_objects_method(self) -> None:
        assert hasattr(OrderingProcessorService, '_reorder_objects')


class TestOrderingService(TestCase):
    def test_has_processor_in_dict(self) -> None:
        assert 'processor' in OrderingService.__dict__

    def test_processor_in_dict_is_ordering_processor_service(self) -> None:
        assert isinstance(OrderingService.__dict__['processor'], OrderingProcessorService)



from __future__ import annotations

import json

from typing import TYPE_CHECKING

from django.http import JsonResponse

from django_spire.ai.chat.intelligence.workflows.chat_workflow import chat_workflow
from django_spire.ai.chat.models import Chat

if TYPE_CHECKING:
    from django.core.handlers.wsgi import WSGIRequest


def delete_chat_view(request: WSGIRequest, pk: int) -> JsonResponse:
    try:
        chat = Chat.objects.get(pk=pk)
    except Chat.DoesNotExist:
        return JsonResponse(
            {
                'type': 'error',
                'message': 'Chat does not exist. Refresh and try again.'
            }
        )

    chat.set_deleted()

    return JsonResponse({'type': 'success', 'message': 'Chat deleted.'})


def rename_chat_view(request: WSGIRequest, pk: int) -> JsonResponse:
    try:
        chat = Chat.objects.get(pk=pk)
    except Chat.DoesNotExist:
        return JsonResponse(
            {
                'type': 'error',
                'message': 'Chat does not exist. Refresh and try again.'
            }
        )

    new_chat_name = json.loads(request.body.decode("utf-8")).get('new_name', '')

    if new_chat_name == '' or len(new_chat_name) > 128:
        return JsonResponse({'type': 'error', 'message': 'Chat name was not updated.'})

    chat.name = new_chat_name
    chat.save()

    return JsonResponse({'type': 'success'})


def chat_workflow_view(request: WSGIRequest) -> JsonResponse:
    body = json.loads(request.body.decode('utf-8'))
    return JsonResponse({'response': chat_workflow(request, body)})



from __future__ import annotations

import pytest

from django.core.exceptions import ValidationError

from django_spire.ai.context.choices import PersonRoleChoices
from django_spire.ai.context.intelligence.prompts.organization_prompts import organization_info_prompt
from django_spire.ai.context.models import Organization, Person
from django_spire.core.tests.test_cases import BaseTestCase


class OrganizationModelTests(BaseTestCase):
    def test_organization_creation(self) -> None:
        org = Organization.objects.create(
            name='Test Organization',
            legal_name='Test Organization Inc.'
        )

        assert org.name == 'Test Organization'
        assert org.legal_name == 'Test Organization Inc.'

    def test_organization_only_one_allowed(self) -> None:
        Organization.objects.create(name='First Org')

        with pytest.raises(ValidationError):
            Organization.objects.create(name='Second Org')

    def test_organization_all_fields(self) -> None:
        org = Organization.objects.create(
            name='Test Org',
            legal_name='Test Org LLC',
            description='A test organization',
            sector='Technology',
            sub_sector='Software',
            website='https://example.com',
            street_address='123 Main St',
            unit_number='Suite 100',
            city='New York',
            province='NY',
            postal_code='10001',
            country='USA',
            phone='555-1234',
            email='info@example.com'
        )

        assert org.name == 'Test Org'
        assert org.sector == 'Technology'
        assert org.city == 'New York'
        assert org.email == 'info@example.com'

    def test_organization_nullable_fields(self) -> None:
        org = Organization.objects.create()

        assert org.name is None
        assert org.legal_name is None
        assert org.description is None

    def test_organization_get_only_or_none_exists(self) -> None:
        Organization.objects.create(name='Test Org')

        result = Organization.objects.get_only_or_none()

        assert result is not None
        assert result.name == 'Test Org'

    def test_organization_get_only_or_none_not_exists(self) -> None:
        result = Organization.objects.get_only_or_none()

        assert result is None


class PersonModelTests(BaseTestCase):
    def test_person_creation(self) -> None:
        person = Person.objects.create(
            first_name='John',
            last_name='Doe',
            role=PersonRoleChoices.ADMIN
        )

        assert person.first_name == 'John'
        assert person.last_name == 'Doe'
        assert person.role == PersonRoleChoices.ADMIN

    def test_person_default_role(self) -> None:
        person = Person.objects.create(
            first_name='Jane',
            last_name='Smith'
        )

        assert person.role == PersonRoleChoices.ADMIN

    def test_person_default_is_internal(self) -> None:
        person = Person.objects.create(
            first_name='Test',
            last_name='User'
        )

        assert person.is_internal_to_organization is True

    def test_person_all_roles(self) -> None:
        for role_choice in PersonRoleChoices:
            person = Person.objects.create(
                first_name=f'Test_{role_choice.value}',
                last_name='User',
                role=role_choice
            )

            assert person.role == role_choice

    def test_person_with_user(self) -> None:
        person = Person.objects.create(
            user=self.super_user,
            first_name='Super',
            last_name='User'
        )

        assert person.user == self.super_user

    def test_person_nullable_fields(self) -> None:
        person = Person.objects.create()

        assert person.first_name is None
        assert person.last_name is None
        assert person.phone is None
        assert person.email is None
        assert person.role_details is None

    def test_person_external_to_organization(self) -> None:
        person = Person.objects.create(
            first_name='External',
            last_name='Person',
            is_internal_to_organization=False
        )

        assert person.is_internal_to_organization is False


class PersonRoleChoicesTests(BaseTestCase):
    def test_admin_choice(self) -> None:
        assert PersonRoleChoices.ADMIN.value == 'admin'
        assert PersonRoleChoices.ADMIN.label == 'Admin'

    def test_human_resources_choice(self) -> None:
        assert PersonRoleChoices.HUMAN_RESOURCES.value == 'human_resources'
        assert PersonRoleChoices.HUMAN_RESOURCES.label == 'Human Resources'

    def test_sales_choice(self) -> None:
        assert PersonRoleChoices.SALES.value == 'sales'
        assert PersonRoleChoices.SALES.label == 'Sales'

    def test_manager_choice(self) -> None:
        assert PersonRoleChoices.MANAGER.value == 'manager'
        assert PersonRoleChoices.MANAGER.label == 'Manager'

    def test_marketing_choice(self) -> None:
        assert PersonRoleChoices.MARKETING.value == 'marketing'
        assert PersonRoleChoices.MARKETING.label == 'Marketing'

    def test_technical_choice(self) -> None:
        assert PersonRoleChoices.TECHNICAL.value == 'technical'
        assert PersonRoleChoices.TECHNICAL.label == 'Technical'

    def test_it_support_choice(self) -> None:
        assert PersonRoleChoices.IT_SUPPORT.value == 'it_support'
        assert PersonRoleChoices.IT_SUPPORT.label == 'IT Support'

    def test_all_choices_count(self) -> None:
        assert len(PersonRoleChoices) == 7


class OrganizationPromptTests(BaseTestCase):
    def test_organization_info_prompt_with_org(self) -> None:
        Organization.objects.create(
            name='Test Org',
            legal_name='Test Org Inc.'
        )

        prompt = organization_info_prompt()

        assert prompt is not None
        assert 'Organization Information' in prompt.to_str()

    def test_organization_info_prompt_without_org(self) -> None:
        prompt = organization_info_prompt()

        assert prompt is not None
        assert 'no organization information available' in prompt.to_str()



from __future__ import annotations

from django_spire.ai.chat.choices import MessageResponseType
from django_spire.ai.chat.message_intel import DefaultMessageIntel
from django_spire.ai.chat.responses import MessageResponse, MessageResponseGroup
from django_spire.core.tests.test_cases import BaseTestCase


class MessageResponseTests(BaseTestCase):
    def test_message_response_creation(self) -> None:
        response = MessageResponse(
            type=MessageResponseType.REQUEST,
            sender='User',
            message_intel=DefaultMessageIntel(text='Hello')
        )

        assert response.type == MessageResponseType.REQUEST
        assert response.sender == 'User'
        assert response.message_intel.text == 'Hello'

    def test_message_response_with_timestamp(self) -> None:
        response = MessageResponse(
            type=MessageResponseType.REQUEST,
            sender='User',
            message_intel=DefaultMessageIntel(text='Hello'),
            message_timestamp='Jan 01, 2024 at 12:00 PM'
        )

        assert response.message_timestamp == 'Jan 01, 2024 at 12:00 PM'

    def test_message_response_with_synthesis_speech(self) -> None:
        response = MessageResponse(
            type=MessageResponseType.RESPONSE,
            sender='Assistant',
            message_intel=DefaultMessageIntel(text='Hello'),
            synthesis_speech=True
        )

        assert response.synthesis_speech is True

    def test_message_response_render_to_html_string_request(self) -> None:
        response = MessageResponse(
            type=MessageResponseType.REQUEST,
            sender='User',
            message_intel=DefaultMessageIntel(text='Hello')
        )

        html = response.render_to_html_string()

        assert isinstance(html, str)
        assert len(html) > 0

    def test_message_response_render_to_html_string_response(self) -> None:
        response = MessageResponse(
            type=MessageResponseType.RESPONSE,
            sender='Assistant',
            message_intel=DefaultMessageIntel(text='Hello')
        )

        html = response.render_to_html_string()

        assert isinstance(html, str)
        assert len(html) > 0

    def test_message_response_render_to_html_string_loading(self) -> None:
        response = MessageResponse(
            type=MessageResponseType.LOADING_RESPONSE,
            sender='Assistant',
            message_intel=DefaultMessageIntel(text='Loading...')
        )

        html = response.render_to_html_string()

        assert isinstance(html, str)
        assert len(html) > 0

    def test_message_response_render_with_context_data(self) -> None:
        response = MessageResponse(
            type=MessageResponseType.REQUEST,
            sender='User',
            message_intel=DefaultMessageIntel(text='Hello')
        )

        html = response.render_to_html_string(context_data={'chat_id': 123})

        assert isinstance(html, str)

    def test_message_response_default_synthesis_speech(self) -> None:
        response = MessageResponse(
            type=MessageResponseType.REQUEST,
            sender='User',
            message_intel=DefaultMessageIntel(text='Hello')
        )

        assert response.synthesis_speech is False

    def test_message_response_default_timestamp(self) -> None:
        response = MessageResponse(
            type=MessageResponseType.REQUEST,
            sender='User',
            message_intel=DefaultMessageIntel(text='Hello')
        )

        assert response.message_timestamp is None


class MessageResponseGroupTests(BaseTestCase):
    def test_message_response_group_creation(self) -> None:
        group = MessageResponseGroup()

        assert len(group.message_responses) == 0

    def test_add_message_response(self) -> None:
        group = MessageResponseGroup()
        response = MessageResponse(
            type=MessageResponseType.REQUEST,
            sender='User',
            message_intel=DefaultMessageIntel(text='Hello')
        )

        group.add_message_response(response)

        assert len(group.message_responses) == 1

    def test_add_multiple_message_responses(self) -> None:
        group = MessageResponseGroup()

        for i in range(5):
            response = MessageResponse(
                type=MessageResponseType.REQUEST,
                sender='User',
                message_intel=DefaultMessageIntel(text=f'Message {i}')
            )
            group.add_message_response(response)

        assert len(group.message_responses) == 5

    def test_render_to_html_string_empty(self) -> None:
        group = MessageResponseGroup()

        html = group.render_to_html_string()

        assert html == ''

    def test_render_to_html_string_with_messages(self) -> None:
        group = MessageResponseGroup()
        group.add_message_response(MessageResponse(
            type=MessageResponseType.REQUEST,
            sender='User',
            message_intel=DefaultMessageIntel(text='Hello')
        ))
        group.add_message_response(MessageResponse(
            type=MessageResponseType.RESPONSE,
            sender='Assistant',
            message_intel=DefaultMessageIntel(text='Hi')
        ))

        html = group.render_to_html_string()

        assert isinstance(html, str)
        assert len(html) > 0

    def test_render_to_html_string_with_context(self) -> None:
        group = MessageResponseGroup()
        group.add_message_response(MessageResponse(
            type=MessageResponseType.REQUEST,
            sender='User',
            message_intel=DefaultMessageIntel(text='Hello')
        ))

        html = group.render_to_html_string(context_data={'chat_id': 123})

        assert isinstance(html, str)

    def test_render_to_html_string_with_is_loading(self) -> None:
        group = MessageResponseGroup()
        group.add_message_response(MessageResponse(
            type=MessageResponseType.REQUEST,
            sender='User',
            message_intel=DefaultMessageIntel(text='Hello')
        ))

        html = group.render_to_html_string(context_data={'is_loading': True})

        assert isinstance(html, str)

    def test_render_to_html_string_default_is_loading(self) -> None:
        group = MessageResponseGroup()
        group.add_message_response(MessageResponse(
            type=MessageResponseType.REQUEST,
            sender='User',
            message_intel=DefaultMessageIntel(text='Hello')
        ))

        html = group.render_to_html_string(context_data={})

        assert isinstance(html, str)


class MessageResponseTypeTests(BaseTestCase):
    def test_request_type(self) -> None:
        assert MessageResponseType.REQUEST.value == 'request'

    def test_response_type(self) -> None:
        assert MessageResponseType.RESPONSE.value == 'response'

    def test_loading_response_type(self) -> None:
        assert MessageResponseType.LOADING_RESPONSE.value == 'loading_response'



from __future__ import annotations

from django.contrib.auth.models import User

from django_spire.notification.choices import (
    NotificationPriorityChoices,
    NotificationStatusChoices,
    NotificationTypeChoices,
)
from django_spire.notification.models import Notification


def create_test_notification(**kwargs) -> Notification:
    if 'user' not in kwargs:
        kwargs['user'] = User.objects.first()

    data = {
        'type': NotificationTypeChoices.APP,
        'title': 'Test Notification',
        'body': 'This is a test notification body.',
        'url': 'https://example.com',
        'status': NotificationStatusChoices.PENDING,
        'priority': NotificationPriorityChoices.LOW,
    }
    data.update(kwargs)
    return Notification.objects.create(**data)



from __future__ import annotations

from django_spire.core.tests.test_cases import BaseTestCase
from django_spire.knowledge.entry.version.choices import EntryVersionStatusChoices


class EntryVersionStatusChoicesTests(BaseTestCase):
    def test_draft_value(self):
        assert EntryVersionStatusChoices.DRAFT == 'draft'

    def test_published_value(self):
        assert EntryVersionStatusChoices.PUBLISHED == 'published'

    def test_archived_value(self):
        assert EntryVersionStatusChoices.ARCHIVED == 'archived'

    def test_choices_count(self):
        assert len(EntryVersionStatusChoices.choices) == 3



from __future__ import annotations

from unittest.mock import MagicMock

from django.test import TestCase

from django_spire.contrib.breadcrumb.breadcrumbs import BreadcrumbItem, Breadcrumbs


class TestBreadcrumbItem(TestCase):
    def test_create_with_href(self) -> None:
        item = BreadcrumbItem(name='Home', href='/home/')

        assert item.name == 'Home'
        assert item.href == '/home/'

    def test_create_without_href(self) -> None:
        item = BreadcrumbItem(name='Current Page')

        assert item.name == 'Current Page'
        assert item.href is None

    def test_to_dict(self) -> None:
        item = BreadcrumbItem(name='Home', href='/home/')
        result = item.to_dict()

        assert result == {'name': 'Home', 'href': '/home/'}

    def test_to_dict_without_href(self) -> None:
        item = BreadcrumbItem(name='Current Page')
        result = item.to_dict()

        assert result == {'name': 'Current Page', 'href': None}


class TestBreadcrumbs(TestCase):
    def test_add_base_breadcrumb(self) -> None:
        breadcrumbs = Breadcrumbs()

        model = MagicMock()
        model_breadcrumbs = Breadcrumbs()
        model_breadcrumbs.add_breadcrumb('Base', '/base/')
        model.base_breadcrumb.return_value = model_breadcrumbs

        breadcrumbs.add_base_breadcrumb(model)

        assert len(breadcrumbs) == 1

    def test_add_base_breadcrumb_no_method(self) -> None:
        breadcrumbs = Breadcrumbs()

        model = MagicMock(spec=[])

        breadcrumbs.add_base_breadcrumb(model)

        assert len(breadcrumbs) == 0

    def test_add_breadcrumb(self) -> None:
        breadcrumbs = Breadcrumbs()
        breadcrumbs.add_breadcrumb('Home', '/home/')

        assert len(breadcrumbs) == 1
        assert breadcrumbs.data[0].name == 'Home'
        assert breadcrumbs.data[0].href == '/home/'

    def test_add_breadcrumb_without_href(self) -> None:
        breadcrumbs = Breadcrumbs()
        breadcrumbs.add_breadcrumb('Current Page')

        assert len(breadcrumbs) == 1
        assert breadcrumbs.data[0].name == 'Current Page'
        assert breadcrumbs.data[0].href is None

    def test_add_form_breadcrumbs_create(self) -> None:
        breadcrumbs = Breadcrumbs()

        obj = MagicMock()
        obj.pk = None
        obj._meta.model._meta.verbose_name = 'Test Model'
        del obj.breadcrumbs

        breadcrumbs.add_form_breadcrumbs(obj)

        assert len(breadcrumbs) == 2
        assert breadcrumbs.data[0].name == 'Test Model'
        assert breadcrumbs.data[1].name == 'Create'

    def test_add_form_breadcrumbs_edit(self) -> None:
        breadcrumbs = Breadcrumbs()

        obj = MagicMock()
        obj.pk = 1
        del obj.breadcrumbs

        breadcrumbs.add_form_breadcrumbs(obj)

        assert len(breadcrumbs) == 1
        assert breadcrumbs.data[0].name == 'Edit'

    def test_add_obj_breadcrumbs(self) -> None:
        breadcrumbs = Breadcrumbs()

        obj_breadcrumbs = Breadcrumbs()
        obj_breadcrumbs.add_breadcrumb('Object', '/object/')

        obj = MagicMock()
        obj.breadcrumbs.return_value = obj_breadcrumbs

        breadcrumbs.add_obj_breadcrumbs(obj)

        assert len(breadcrumbs) == 1
        assert breadcrumbs.data[0].name == 'Object'

    def test_add_obj_breadcrumbs_no_method(self) -> None:
        breadcrumbs = Breadcrumbs()

        obj = MagicMock(spec=[])

        breadcrumbs.add_obj_breadcrumbs(obj)

        assert len(breadcrumbs) == 0

    def test_add_operator(self) -> None:
        breadcrumbs1 = Breadcrumbs()
        breadcrumbs1.add_breadcrumb('Home', '/home/')

        breadcrumbs2 = Breadcrumbs()
        breadcrumbs2.add_breadcrumb('Page', '/page/')

        result = breadcrumbs1 + breadcrumbs2

        assert len(result) == 2
        assert result.data[0].name == 'Home'
        assert result.data[1].name == 'Page'

    def test_empty_breadcrumbs(self) -> None:
        breadcrumbs = Breadcrumbs()

        assert len(breadcrumbs) == 0

    def test_iteration(self) -> None:
        breadcrumbs = Breadcrumbs()
        breadcrumbs.add_breadcrumb('Home', '/home/')
        breadcrumbs.add_breadcrumb('Page', '/page/')

        items = list(breadcrumbs)

        assert len(items) == 2
        assert items[0] == {'name': 'Home', 'href': '/home/'}
        assert items[1] == {'name': 'Page', 'href': '/page/'}

    def test_iteration_empty(self) -> None:
        breadcrumbs = Breadcrumbs()

        items = list(breadcrumbs)

        assert items == []

    def test_len(self) -> None:
        breadcrumbs = Breadcrumbs()
        breadcrumbs.add_breadcrumb('Home', '/home/')
        breadcrumbs.add_breadcrumb('Page', '/page/')

        assert len(breadcrumbs) == 2

    def test_remove(self) -> None:
        breadcrumbs = Breadcrumbs()
        breadcrumbs.add_breadcrumb('Home', '/home/')
        breadcrumbs.add_breadcrumb('Page', '/page/')
        breadcrumbs.add_breadcrumb('Current', None)

        result = breadcrumbs.remove(1)

        assert len(breadcrumbs) == 2
        assert breadcrumbs.data[0].name == 'Home'
        assert breadcrumbs.data[1].name == 'Current'
        assert result is breadcrumbs

    def test_reverse(self) -> None:
        breadcrumbs = Breadcrumbs()
        breadcrumbs.add_breadcrumb('First', '/first/')
        breadcrumbs.add_breadcrumb('Second', '/second/')
        breadcrumbs.add_breadcrumb('Third', '/third/')

        result = breadcrumbs.reverse()

        assert breadcrumbs.data[0].name == 'Third'
        assert breadcrumbs.data[1].name == 'Second'
        assert breadcrumbs.data[2].name == 'First'
        assert result is breadcrumbs

    def test_str(self) -> None:
        breadcrumbs = Breadcrumbs()
        breadcrumbs.add_breadcrumb('Home', '/home/')

        result = str(breadcrumbs)

        assert 'BreadcrumbItem' in result or '[' in result



from __future__ import annotations

from typing import TYPE_CHECKING

from django_spire.exceptions import DjangoSpireError

if TYPE_CHECKING:
    from django_spire.help_desk.enums import TicketEventType
    from django_spire.notification.choices import NotificationTypeChoices


class HelpDeskError(DjangoSpireError):
    pass


class TicketEventNotificationTypeNotSupportedError(HelpDeskError, TypeError):
    def __init__(
        self,
        event_type: TicketEventType | None,
        notification_type: NotificationTypeChoices
    ) -> None:
        if event_type is None:
            super().__init__(f'Notification type not supported: {notification_type}')
        else:
            super().__init__(
                f'Combination of event type and notification type not supported: '
                f'Event type {event_type} - Notification type {notification_type}'
            )


class HelpDeskNotificationRecipientMissingEmailError(HelpDeskError, ValueError):
    def __init__(self, recipient: str) -> None:
        super().__init__(f'Recipient is missing an email address: {recipient}')



from __future__ import annotations

from unittest.mock import MagicMock, patch

from django_spire.core.tests.test_cases import BaseTestCase
from django_spire.notification.managers import NotificationManager


class NotificationManagerTests(BaseTestCase):
    @patch('django_spire.notification.managers.NotificationProcessor')
    def test_process_ready_notifications(self, mock_processor_class: MagicMock):
        mock_processor = MagicMock()
        mock_processor_class.return_value = mock_processor

        NotificationManager.process_ready_notifications()

        mock_processor.process_ready.assert_called_once()

    @patch('django_spire.notification.managers.NotificationProcessor')
    def test_process_errored_notifications(self, mock_processor_class: MagicMock):
        mock_processor = MagicMock()
        mock_processor_class.return_value = mock_processor

        NotificationManager.process_errored_notifications()

        mock_processor.process_errored.assert_called_once()

    @patch('django_spire.notification.managers.AppNotificationProcessor')
    def test_process_ready_app_notifications(self, mock_processor_class: MagicMock):
        mock_processor = MagicMock()
        mock_processor_class.return_value = mock_processor

        NotificationManager.process_ready_app_notifications()

        mock_processor.process_ready.assert_called_once()

    @patch('django_spire.notification.managers.AppNotificationProcessor')
    def test_process_errored_app_notifications(self, mock_processor_class: MagicMock):
        mock_processor = MagicMock()
        mock_processor_class.return_value = mock_processor

        NotificationManager.process_errored_app_notifications()

        mock_processor.process_errored.assert_called_once()

    @patch('django_spire.notification.managers.EmailNotificationProcessor')
    def test_process_ready_email_notifications(self, mock_processor_class: MagicMock):
        mock_processor = MagicMock()
        mock_processor_class.return_value = mock_processor

        NotificationManager.process_ready_email_notifications()

        mock_processor.process_ready.assert_called_once()

    @patch('django_spire.notification.managers.EmailNotificationProcessor')
    def test_process_errored_email_notifications(self, mock_processor_class: MagicMock):
        mock_processor = MagicMock()
        mock_processor_class.return_value = mock_processor

        NotificationManager.process_errored_email_notifications()

        mock_processor.process_errored.assert_called_once()

    @patch('django_spire.notification.managers.SMSNotificationProcessor')
    def test_process_ready_sms_notifications(self, mock_processor_class: MagicMock):
        mock_processor = MagicMock()
        mock_processor_class.return_value = mock_processor

        NotificationManager.process_ready_sms_notifications()

        mock_processor.process_ready.assert_called_once()

    @patch('django_spire.notification.managers.SMSNotificationProcessor')
    def test_process_errored_sms_notifications(self, mock_processor_class: MagicMock):
        mock_processor = MagicMock()
        mock_processor_class.return_value = mock_processor

        NotificationManager.process_errored_sms_notifications()

        mock_processor.process_errored.assert_called_once()

    @patch('django_spire.notification.managers.NotificationProcessor')
    def test_process_notification(self, mock_processor_class: MagicMock):
        mock_processor = MagicMock()
        mock_processor_class.return_value = mock_processor
        mock_notification = MagicMock()

        NotificationManager.process_notification(mock_notification)

        mock_processor.process.assert_called_once_with(mock_notification)

    @patch('django_spire.notification.managers.NotificationProcessor')
    def test_process_notifications(self, mock_processor_class: MagicMock):
        mock_processor = MagicMock()
        mock_processor_class.return_value = mock_processor
        mock_notifications = [MagicMock(), MagicMock()]

        NotificationManager.process_notifications(mock_notifications)

        mock_processor.process_list.assert_called_once_with(mock_notifications)



from __future__ import annotations

from django.contrib import admin
from django.urls import reverse
from django.utils.html import format_html

from django_spire.comment import models


@admin.register(models.Comment)
class CommentAdmin(admin.ModelAdmin):
    list_display = (
        'id', 'user_link', 'content_object_link', 'information_snippet',
        'created_datetime', 'is_edited'
    )
    list_filter = ('created_datetime', 'is_edited')
    search_fields = ('id', 'user__username', 'information')
    ordering = ('-created_datetime',)

    def user_link(self, comment: models.Comment) -> str:
        url = reverse('admin:auth_user_change', args=[comment.user.id])
        return format_html(f'<a href="{url}">{comment.user.username}</a>')

    user_link.short_description = 'User'

    def content_object_link(self, comment: models.Comment) -> str:
        url = (
            reverse(
                f'admin:{comment.content_type.app_label}_{comment.content_type.model}_change',
                args=[comment.object_id]
            )
        )

        return format_html(f'<a href="{url}">{comment.content_object}</a>')

    content_object_link.short_description = 'Content Object'

    def information_snippet(self, comment: models.Comment) -> str:
        return (
            comment.information[:20] + '...'
            if len(comment.information) > 20
            else comment.information
        )

    information_snippet.short_description = 'Comment Snippet'



from __future__ import annotations

from django_spire.auth.group.models import AuthGroup
from django_spire.auth.user.forms import RegisterUserForm, UserForm, UserGroupForm
from django_spire.auth.user.models import AuthUser
from django_spire.auth.user.tests.factories import create_user
from django_spire.core.tests.test_cases import BaseTestCase


class UserFormTestCase(BaseTestCase):
    def setUp(self) -> None:
        super().setUp()

        self.user = create_user(
            username='testuser',
            first_name='Test',
            last_name='User',
            email='test@example.com'
        )

    def test_valid_form(self) -> None:
        form = UserForm(
            data={
                'first_name': 'Updated',
                'last_name': 'Name',
                'email': 'updated@example.com',
                'is_active': True
            },
            instance=self.user
        )
        assert form.is_valid()

    def test_save_updates_username_to_email(self) -> None:
        form = UserForm(
            data={
                'first_name': 'Test',
                'last_name': 'User',
                'email': 'newemail@example.com',
                'is_active': True
            },
            instance=self.user
        )
        assert form.is_valid()
        user = form.save()
        assert user.username == 'newemail@example.com'

    def test_empty_first_name(self) -> None:
        form = UserForm(
            data={
                'first_name': '',
                'last_name': 'User',
                'email': 'test@example.com',
                'is_active': True
            },
            instance=self.user
        )
        assert form.is_valid()

    def test_empty_last_name(self) -> None:
        form = UserForm(
            data={
                'first_name': 'Test',
                'last_name': '',
                'email': 'test@example.com',
                'is_active': True
            },
            instance=self.user
        )
        assert form.is_valid()

    def test_invalid_email(self) -> None:
        form = UserForm(
            data={
                'first_name': 'Test',
                'last_name': 'User',
                'email': 'not-an-email',
                'is_active': True
            },
            instance=self.user
        )
        assert not form.is_valid()
        assert 'email' in form.errors

    def test_empty_email(self) -> None:
        form = UserForm(
            data={
                'first_name': 'Test',
                'last_name': 'User',
                'email': '',
                'is_active': True
            },
            instance=self.user
        )
        assert form.is_valid()

    def test_is_active_false(self) -> None:
        form = UserForm(
            data={
                'first_name': 'Test',
                'last_name': 'User',
                'email': 'test@example.com',
                'is_active': False
            },
            instance=self.user
        )
        assert form.is_valid()
        user = form.save()
        assert not user.is_active

    def test_unicode_names(self) -> None:
        form = UserForm(
            data={
                'first_name': 'Tëst',
                'last_name': 'Üser',
                'email': 'test@example.com',
                'is_active': True
            },
            instance=self.user
        )
        assert form.is_valid()

    def test_form_fields(self) -> None:
        form = UserForm()
        assert 'first_name' in form.fields
        assert 'last_name' in form.fields
        assert 'email' in form.fields
        assert 'is_active' in form.fields

    def test_form_excludes_password(self) -> None:
        form = UserForm()
        assert 'password' not in form.fields

    def test_form_excludes_username(self) -> None:
        form = UserForm()
        assert 'username' not in form.fields

    def test_save_preserves_password(self) -> None:
        self.user.set_password('originalpassword')
        self.user.save()
        form = UserForm(
            data={
                'first_name': 'Updated',
                'last_name': 'User',
                'email': 'updated@example.com',
                'is_active': True
            },
            instance=self.user
        )
        form.is_valid()
        user = form.save()
        assert user.check_password('originalpassword')

    def test_email_with_plus_sign(self) -> None:
        form = UserForm(
            data={
                'first_name': 'Test',
                'last_name': 'User',
                'email': 'test+alias@example.com',
                'is_active': True
            },
            instance=self.user
        )
        assert form.is_valid()

    def test_long_first_name(self) -> None:
        form = UserForm(
            data={
                'first_name': 'A' * 150,
                'last_name': 'User',
                'email': 'test@example.com',
                'is_active': True
            },
            instance=self.user
        )
        assert form.is_valid()

    def test_long_last_name(self) -> None:
        form = UserForm(
            data={
                'first_name': 'Test',
                'last_name': 'A' * 150,
                'email': 'test@example.com',
                'is_active': True
            },
            instance=self.user
        )
        assert form.is_valid()

    def test_first_name_with_hyphen(self) -> None:
        form = UserForm(
            data={
                'first_name': 'Mary-Jane',
                'last_name': 'Watson',
                'email': 'test@example.com',
                'is_active': True
            },
            instance=self.user
        )
        assert form.is_valid()

    def test_last_name_with_apostrophe(self) -> None:
        form = UserForm(
            data={
                'first_name': 'Patrick',
                'last_name': "O'Brien",
                'email': 'test@example.com',
                'is_active': True
            },
            instance=self.user
        )
        assert form.is_valid()

    def test_email_case_preserved(self) -> None:
        form = UserForm(
            data={
                'first_name': 'Test',
                'last_name': 'User',
                'email': 'Test@Example.COM',
                'is_active': True
            },
            instance=self.user
        )
        assert form.is_valid()
        user = form.save()
        assert user.username == 'Test@Example.COM'


class RegisterUserFormTestCase(BaseTestCase):
    def test_valid_form(self) -> None:
        form = RegisterUserForm(
            data={
                'first_name': 'New',
                'last_name': 'User',
                'email': 'newuser@example.com',
                'password': 'securepassword123'
            }
        )
        assert form.is_valid()

    def test_password_too_short(self) -> None:
        form = RegisterUserForm(
            data={
                'first_name': 'New',
                'last_name': 'User',
                'email': 'newuser@example.com',
                'password': 'short'
            }
        )
        assert not form.is_valid()
        assert 'password' in form.errors

    def test_password_exactly_eight_characters(self) -> None:
        form = RegisterUserForm(
            data={
                'first_name': 'New',
                'last_name': 'User',
                'email': 'newuser@example.com',
                'password': '12345678'
            }
        )
        assert form.is_valid()

    def test_password_seven_characters(self) -> None:
        form = RegisterUserForm(
            data={
                'first_name': 'New',
                'last_name': 'User',
                'email': 'newuser@example.com',
                'password': '1234567'
            }
        )
        assert not form.is_valid()

    def test_save_creates_user(self) -> None:
        form = RegisterUserForm(
            data={
                'first_name': 'New',
                'last_name': 'User',
                'email': 'newuser@example.com',
                'password': 'securepassword123'
            }
        )
        assert form.is_valid()
        user = form.save()
        assert isinstance(user, AuthUser)
        assert user.first_name == 'New'
        assert user.last_name == 'User'
        assert user.email == 'newuser@example.com'
        assert user.username == 'newuser@example.com'

    def test_email_saved_lowercase(self) -> None:
        form = RegisterUserForm(
            data={
                'first_name': 'New',
                'last_name': 'User',
                'email': 'NewUser@Example.COM',
                'password': 'securepassword123'
            }
        )
        assert form.is_valid()
        user = form.save()
        assert user.email == 'newuser@example.com'
        assert user.username == 'newuser@example.com'

    def test_empty_first_name(self) -> None:
        form = RegisterUserForm(
            data={
                'first_name': '',
                'last_name': 'User',
                'email': 'newuser@example.com',
                'password': 'securepassword123'
            }
        )
        assert form.is_valid()

    def test_empty_last_name(self) -> None:
        form = RegisterUserForm(
            data={
                'first_name': 'New',
                'last_name': '',
                'email': 'newuser@example.com',
                'password': 'securepassword123'
            }
        )
        assert form.is_valid()

    def test_invalid_email_format(self) -> None:
        form = RegisterUserForm(
            data={
                'first_name': 'New',
                'last_name': 'User',
                'email': 'invalid-email',
                'password': 'securepassword123'
            }
        )
        assert not form.is_valid()

    def test_empty_email(self) -> None:
        form = RegisterUserForm(
            data={
                'first_name': 'New',
                'last_name': 'User',
                'email': '',
                'password': 'securepassword123'
            }
        )
        assert not form.is_valid()
        assert 'email' in form.errors

    def test_empty_password(self) -> None:
        form = RegisterUserForm(
            data={
                'first_name': 'New',
                'last_name': 'User',
                'email': 'newuser@example.com',
                'password': ''
            }
        )
        assert not form.is_valid()

    def test_password_with_spaces(self) -> None:
        form = RegisterUserForm(
            data={
                'first_name': 'New',
                'last_name': 'User',
                'email': 'newuser@example.com',
                'password': 'pass word 123'
            }
        )
        assert form.is_valid()

    def test_long_password(self) -> None:
        form = RegisterUserForm(
            data={
                'first_name': 'New',
                'last_name': 'User',
                'email': 'newuser@example.com',
                'password': 'a' * 100
            }
        )
        assert form.is_valid()

    def test_created_user_can_authenticate(self) -> None:
        form = RegisterUserForm(
            data={
                'first_name': 'New',
                'last_name': 'User',
                'email': 'newuser@example.com',
                'password': 'securepassword123'
            }
        )
        assert form.is_valid()
        user = form.save()
        assert user.check_password('securepassword123')

    def test_password_error_message(self) -> None:
        form = RegisterUserForm(
            data={
                'first_name': 'New',
                'last_name': 'User',
                'email': 'newuser@example.com',
                'password': 'short'
            }
        )
        form.is_valid()
        assert '8 characters' in form.errors['password'][0]

    def test_form_fields(self) -> None:
        form = RegisterUserForm()
        assert 'first_name' in form.fields
        assert 'last_name' in form.fields
        assert 'email' in form.fields
        assert 'password' in form.fields

    def test_password_with_special_characters(self) -> None:
        form = RegisterUserForm(
            data={
                'first_name': 'New',
                'last_name': 'User',
                'email': 'newuser@example.com',
                'password': 'P@ssw0rd!#$%'
            }
        )
        assert form.is_valid()

    def test_password_with_unicode(self) -> None:
        form = RegisterUserForm(
            data={
                'first_name': 'New',
                'last_name': 'User',
                'email': 'newuser@example.com',
                'password': 'pässwörd日本語'
            }
        )
        assert form.is_valid()

    def test_unicode_names(self) -> None:
        form = RegisterUserForm(
            data={
                'first_name': 'Tëst',
                'last_name': 'Üsér',
                'email': 'newuser@example.com',
                'password': 'securepassword123'
            }
        )
        assert form.is_valid()
        user = form.save()
        assert user.first_name == 'Tëst'
        assert user.last_name == 'Üsér'

    def test_user_is_active_after_registration(self) -> None:
        form = RegisterUserForm(
            data={
                'first_name': 'New',
                'last_name': 'User',
                'email': 'newuser@example.com',
                'password': 'securepassword123'
            }
        )
        form.is_valid()
        user = form.save()
        assert user.is_active

    def test_user_is_not_staff_after_registration(self) -> None:
        form = RegisterUserForm(
            data={
                'first_name': 'New',
                'last_name': 'User',
                'email': 'newuser@example.com',
                'password': 'securepassword123'
            }
        )
        form.is_valid()
        user = form.save()
        assert not user.is_staff

    def test_user_is_not_superuser_after_registration(self) -> None:
        form = RegisterUserForm(
            data={
                'first_name': 'New',
                'last_name': 'User',
                'email': 'newuser@example.com',
                'password': 'securepassword123'
            }
        )
        form.is_valid()
        user = form.save()
        assert not user.is_superuser


class UserGroupFormTestCase(BaseTestCase):
    def setUp(self) -> None:
        super().setUp()
        self.group1 = AuthGroup.objects.create(name='Group 1')
        self.group2 = AuthGroup.objects.create(name='Group 2')
        self.group3 = AuthGroup.objects.create(name='Group 3')

    def test_valid_form(self) -> None:
        form = UserGroupForm(data={'group_list': [self.group1.pk, self.group2.pk]})
        assert form.is_valid()

    def test_cleaned_data_contains_groups(self) -> None:
        form = UserGroupForm(data={'group_list': [self.group1.pk]})
        assert form.is_valid()
        assert self.group1 in form.cleaned_data['group_list']

    def test_single_group(self) -> None:
        form = UserGroupForm(data={'group_list': [self.group1.pk]})
        assert form.is_valid()
        assert len(form.cleaned_data['group_list']) == 1

    def test_multiple_groups(self) -> None:
        form = UserGroupForm(data={'group_list': [self.group1.pk, self.group2.pk, self.group3.pk]})
        assert form.is_valid()
        assert len(form.cleaned_data['group_list']) == 3

    def test_invalid_group_id(self) -> None:
        form = UserGroupForm(data={'group_list': [99999]})
        assert not form.is_valid()

    def test_mixed_valid_invalid_groups(self) -> None:
        form = UserGroupForm(data={'group_list': [self.group1.pk, 99999]})
        assert not form.is_valid()

    def test_empty_group_list_fails(self) -> None:
        form = UserGroupForm(data={'group_list': []})
        assert not form.is_valid()

    def test_no_data_fails(self) -> None:
        form = UserGroupForm(data={})
        assert not form.is_valid()

    def test_duplicate_group_ids(self) -> None:
        form = UserGroupForm(data={'group_list': [self.group1.pk, self.group1.pk]})
        assert form.is_valid()
        assert len(form.cleaned_data['group_list']) == 1

    def test_negative_group_id(self) -> None:
        form = UserGroupForm(data={'group_list': [-1]})
        assert not form.is_valid()

    def test_zero_group_id(self) -> None:
        form = UserGroupForm(data={'group_list': [0]})
        assert not form.is_valid()

    def test_string_group_id(self) -> None:
        form = UserGroupForm(data={'group_list': ['invalid']})
        assert not form.is_valid()

    def test_form_queryset(self) -> None:
        form = UserGroupForm()
        queryset = form.fields['group_list'].queryset
        assert self.group1 in queryset
        assert self.group2 in queryset
        assert self.group3 in queryset

    def test_all_groups_selectable(self) -> None:
        form = UserGroupForm(data={'group_list': [self.group1.pk, self.group2.pk, self.group3.pk]})
        assert form.is_valid()
        groups = form.cleaned_data['group_list']
        assert self.group1 in groups
        assert self.group2 in groups
        assert self.group3 in groups

    def test_group_list_field_required(self) -> None:
        form = UserGroupForm()
        assert form.fields['group_list'].required

    def test_form_with_deleted_group(self) -> None:
        group_pk = self.group3.pk
        self.group3.delete()
        form = UserGroupForm(data={'group_list': [group_pk]})
        assert not form.is_valid()



from __future__ import annotations

import json

from django.test import RequestFactory, override_settings

from django_spire.core.tests.test_cases import BaseTestCase
from django_spire.file.models import File
from django_spire.file.tests.factories import create_test_in_memory_uploaded_file
from django_spire.file.views import file_multiple_upload_ajax, file_single_upload_ajax


STORAGES_OVERRIDE = {
    'default': {
        'BACKEND': 'django.core.files.storage.FileSystemStorage',
    },
    'staticfiles': {
        'BACKEND': 'django.contrib.staticfiles.storage.StaticFilesStorage',
    },
}


@override_settings(STORAGES=STORAGES_OVERRIDE)
class FileMultipleUploadAjaxViewTests(BaseTestCase):
    def setUp(self):
        super().setUp()

        self.factory = RequestFactory()

    def test_post_creates_files(self):
        initial_count = File.objects.count()
        uploaded_file = create_test_in_memory_uploaded_file()

        request = self.factory.post(
            '/upload/multiple/ajax',
            data={'related_field': ''},
        )
        request.FILES['file'] = uploaded_file
        request.user = self.super_user

        file_multiple_upload_ajax(request)

        assert File.objects.count() == initial_count + 1

    def test_post_returns_json_response(self):
        uploaded_file = create_test_in_memory_uploaded_file()

        request = self.factory.post(
            '/upload/multiple/ajax',
            data={'related_field': ''},
        )
        request.FILES['file'] = uploaded_file
        request.user = self.super_user

        response = file_multiple_upload_ajax(request)

        assert response.status_code == 200
        assert response['Content-Type'] == 'application/json'

    def test_post_returns_files_in_response(self):
        uploaded_file = create_test_in_memory_uploaded_file(name='test_upload')

        request = self.factory.post(
            '/upload/multiple/ajax',
            data={'related_field': ''},
        )
        request.FILES['file'] = uploaded_file
        request.user = self.super_user

        response = file_multiple_upload_ajax(request)
        data = json.loads(response.content)

        assert 'files' in data
        assert len(data['files']) == 1
        assert data['files'][0]['name'] == 'test_upload'

    def test_get_returns_none(self):
        request = self.factory.get('/upload/multiple/ajax')
        request.user = self.super_user

        response = file_multiple_upload_ajax(request)

        assert response is None


@override_settings(STORAGES=STORAGES_OVERRIDE)
class FileSingleUploadAjaxViewTests(BaseTestCase):
    def setUp(self):
        super().setUp()

        self.factory = RequestFactory()

    def test_post_creates_file(self):
        initial_count = File.objects.count()
        uploaded_file = create_test_in_memory_uploaded_file()

        request = self.factory.post(
            '/upload/single/ajax',
            data={'related_field': ''},
        )
        request.FILES['file'] = uploaded_file
        request.user = self.super_user

        file_single_upload_ajax(request)

        assert File.objects.count() == initial_count + 1

    def test_post_returns_json_response(self):
        uploaded_file = create_test_in_memory_uploaded_file()

        request = self.factory.post(
            '/upload/single/ajax',
            data={'related_field': ''},
        )
        request.FILES['file'] = uploaded_file
        request.user = self.super_user

        response = file_single_upload_ajax(request)

        assert response.status_code == 200
        assert response['Content-Type'] == 'application/json'

    def test_post_returns_file_in_response(self):
        uploaded_file = create_test_in_memory_uploaded_file(name='single_upload')

        request = self.factory.post(
            '/upload/single/ajax',
            data={'related_field': ''},
        )
        request.FILES['file'] = uploaded_file
        request.user = self.super_user

        response = file_single_upload_ajax(request)
        data = json.loads(response.content)

        assert 'file' in data
        assert data['file']['name'] == 'single_upload'

    def test_get_returns_none(self):
        request = self.factory.get('/upload/single/ajax')
        request.user = self.super_user

        response = file_single_upload_ajax(request)

        assert response is None



from __future__ import annotations

from django.apps import AppConfig

from django_spire.utils import check_required_apps


class UserAccountConfig(AppConfig):
    default_auto_field = 'django.db.models.BigAutoField'
    label = 'django_spire_auth_user'
    name = 'django_spire.auth.user'

    MODEL_PERMISSIONS = (
        {
            'name': 'user',
            'model_class_path': 'django_spire.auth.user.models.AuthUser',
            'is_proxy_model': True
        },
    )

    REQUIRED_APPS = ('django_spire_core', 'django_spire_auth')

    def ready(self) -> None:
        check_required_apps(self.label)



from __future__ import annotations

from django_spire.core.tests.test_cases import BaseTestCase
from django_spire.knowledge.entry.version.block.data.list.choices import OrderedListCounterType
from django_spire.knowledge.entry.version.block.data.list.meta import (
    ChecklistItemMeta,
    OrderedListItemMeta,
)


class ChecklistItemMetaTests(BaseTestCase):
    def test_default_unchecked(self):
        meta = ChecklistItemMeta()
        assert meta.checked is False

    def test_checked_true(self):
        meta = ChecklistItemMeta(checked=True)
        assert meta.checked is True

    def test_checked_false(self):
        meta = ChecklistItemMeta(checked=False)
        assert meta.checked is False


class OrderedListItemMetaTests(BaseTestCase):
    def test_default_values(self):
        meta = OrderedListItemMeta()
        assert meta.start is None
        assert meta.counterType is None

    def test_with_start(self):
        meta = OrderedListItemMeta(start=5)
        assert meta.start == 5

    def test_with_counter_type_numeric(self):
        meta = OrderedListItemMeta(counterType=OrderedListCounterType.NUMERIC)
        assert meta.counterType == OrderedListCounterType.NUMERIC

    def test_with_counter_type_upper_roman(self):
        meta = OrderedListItemMeta(counterType=OrderedListCounterType.UPPER_ROMAN)
        assert meta.counterType == OrderedListCounterType.UPPER_ROMAN

    def test_with_counter_type_lower_roman(self):
        meta = OrderedListItemMeta(counterType=OrderedListCounterType.LOWER_ROMAN)
        assert meta.counterType == OrderedListCounterType.LOWER_ROMAN

    def test_with_counter_type_upper_alpha(self):
        meta = OrderedListItemMeta(counterType=OrderedListCounterType.UPPER_ALPHA)
        assert meta.counterType == OrderedListCounterType.UPPER_ALPHA

    def test_with_counter_type_lower_alpha(self):
        meta = OrderedListItemMeta(counterType=OrderedListCounterType.LOWER_ALPHA)
        assert meta.counterType == OrderedListCounterType.LOWER_ALPHA

    def test_with_all_values(self):
        meta = OrderedListItemMeta(start=10, counterType=OrderedListCounterType.NUMERIC)
        assert meta.start == 10
        assert meta.counterType == OrderedListCounterType.NUMERIC
